'''implements a function to create a diagram with data drawn over index

This module implements a function to create a diagram with the data drawn over the index in the data set.
It is used to display the development of a optimization result over time or better over the iteration index of the algorithm.

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
'''

import numpy
from os import path
from time import strftime
import matplotlib.pyplot as pylab
#from dsc_suite.analyses.standards import SIZES
#from dsc_suite.analyses.standards import AXES

from dsc_suite.tools.toolpaths import FOLDER_FOR_DATA, FOLDER_FOR_PLOTS


def plot_costdata_development(filenames, information, parameter, create_png=False):
    '''Plots the cost data in order of their index in file
    
    This function creates a diagram with the data plotted in order of their index in the data file.
    It creates a new figure and gets the information about the data files from the information dictionary.
    The diagram is adjusted by the entries of the dictionary parameters.
    
    @param filenames: filenames relative to the default folder of data
    @type filenames: list
    @param information: information about the files in filenames extracted from XML data base
    @type information: dictionary
    @param parameter: parameters for the diagram layout
    @type parameter: dictionary
    @param create_png: create a png file(True) from diagram or show interactive mode(False)
    @type create_png: boolean
    '''
    #get different dictionaries
    labels = parameter["labels"]
    sizes = parameter["sizes"]
    axes = parameter["axes"]
    
    #get information about lables
    title = labels["title"]
    legend_position = labels["legend-position"]
    legend_title = labels["legend-title"]
    legend_parameter = labels["legend-parameter"]
    xlabel = labels["x-label"]
    ylabel = labels["y-label"]
    
    #get information about size
    width = sizes["dimensions"]["width"]
    length = sizes["dimensions"]["length"]
    left = sizes["margins"]["left"]
    right = sizes["margins"]["right"]
    top = sizes["margins"]["top"]
    bottom = sizes["margins"]["bottom"]
    
    #get information about axes
    if axes["x-axe"]["xmin"] == "auto":
        xmin = 0.0
    else:
        xmin = float(axes["x-axe"]["xmin"])
    if axes["x-axe"]["xmax"] == "auto":
        xsearch = True
        xmax = 0.0
    else:
        xmax = float(axes["x-axe"]["xmax"])
        xsearch = False
    if axes["y-axe"]["ymin"] == "auto":
        ymin = 0.0
    else:
        ymin = float(axes["y-axe"]["ymin"])
    if axes["y-axe"]["ymax"] == "auto":
        ymax = 0.0
        ysearch= True 
    else:
        ymax = float(axes["y-axe"]["ymax"])
        ysearch = False
        
    #create figure
    pylab.figure(1,(length,width))
    pylab.subplots_adjust(left,bottom,right,top)
    pylab.ticklabel_format(style = "sci", scilimits = (-2,5))
    
    #prepare plot data
    yarrays = []
    xarrays = []
    legend_labels = []
    print_options = []
    line_styles = []
    highest_value = 0
    longest_iteration = 0
    #load files
    for file in filenames:
        load_array = numpy.load(path.join(FOLDER_FOR_DATA, file))
        if len(load_array) > longest_iteration:
            longest_iteration = len(load_array)
        if max(load_array) > highest_value:
            highest_value = max(load_array)*1.01
        yarrays.append(load_array)
        xarrays.append(range(0,len(load_array),1))
        try:
            algorithm = information[file]["algorithm"]
        except:
            algorithm = None
        if algorithm == "Evolutionary Algorithm":
            print_options.append(".")
            line_styles.append("None")
        else:
            print_options.append("-")
            line_styles.append("-")
        try:
            legend_labels.append(information[file][legend_parameter])
        except:
            try: 
                legend_labels.append(information[file][legend_parameter.replace(" ", "_")])
            except:
                for key in information[file]["cost_criteria"]:
                    try:
                        legend_labels.append(information[file]["cost_criteria"][key][legend_parameter])
                    except:
                        try:
                            legend_labels.append(information[file]["cost_criteria"][key][legend_parameter.replace(" ","_")])
                        except:
                            pass                       
    if xsearch:
        xmax = longest_iteration
    if ysearch:
        ymax = highest_value 
    #plot data
    if len(legend_labels) > 0:
        for i in range(0,len(yarrays),1):
            pylab.plot(xarrays[i], yarrays[i], label = legend_labels[i], linestyle=line_styles[i], marker=print_options[i])
    else:
        for i in range(0,len(yarrays),1):
            pylab.plot(xarrays[i], yarrays[i], linestyle=line_styles[i], marker=print_options[i])
    pylab.xlabel(xlabel)
    pylab.ylabel(ylabel)  
    #set axis limits   
    pylab.axis([xmin,xmax,ymin,ymax])
    #set diagram title
    pylab.suptitle(title)
    #create legend
    if len(legend_labels) > 0:
        pylab.legend(title = legend_title, loc=legend_position)
    #enable grid
    pylab.grid(True)
    #save png file
    if create_png:
        i = 1
        filename = 'Cost development %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        while path.exists(FOLDER_FOR_PLOTS + '/' + filename):
            i += 1
            filename = 'Cost development %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        pylab.savefig(FOLDER_FOR_PLOTS + '/' + filename)
        pylab.close()
        return filename
    else:
        pylab.show()