'''implements a function to plot the distribution of data in a data set

This module implements a function to plot the distribution of data in a data set.
It is used to evaluate a possible distribution of a single cost criteria and should be used Monte Carlo searches

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
'''

import numpy
import pylab
from dsc_suite.tools.toolpaths import FOLDER_FOR_DATA, FOLDER_FOR_PLOTS
from matplotlib.colors import rgb2hex
from colorsys import hsv_to_rgb
from time import strftime
from os import path
from scipy import histogram


#TODO: Labels, legend, title, normalize SEE TRIAL_20100907
def plot_cost_distribution(filenames, information, parameter, create_png=False):
    """Plots a cost distribution.
    
    Plots the distribution of one cost criteria. Useful to compare one benchmark represented by different data structures.
    
    @param filenames: filenames relative to the default folder of data
    @type filenames: list
    @param information: information about the files in filenames extracted from XML data base
    @type information: dictionary
    @param parameter: parameters for the diagram layout
    @type parameter: dictionary
    @param create_png: create a png file(True) from diagram or show interactive mode(False)
    @type create_png: boolean   
    """
    num_bins = 500
    #get different dictionaries
    labels = parameter["labels"]
    sizes = parameter["sizes"]
    axes = parameter["axes"]
    
    #get information about lables
    title = labels["title"]
    legend_position = labels["legend-position"]
    legend_title = labels["legend-title"]
    legend_parameter = labels["legend-parameter"]
    xlabel = labels["x-label"]
    ylabel = labels["y-label"]
    
    #get information about size
    width = sizes["dimensions"]["width"]
    length = sizes["dimensions"]["length"]
    left = sizes["margins"]["left"]
    right = sizes["margins"]["right"]
    top = sizes["margins"]["top"]
    bottom = sizes["margins"]["bottom"]
    
    #get information about axes
    if axes["x-axe"]["xmin"] == "auto":
        xmin = 0.0
    else:
        xmin = float(axes["x-axe"]["xmin"])
    if axes["x-axe"]["xmax"] == "auto":
        xsearch = True
        xmax = 0.0
    else:
        xmax = float(axes["x-axe"]["xmax"])
        xsearch = False
    if axes["y-axe"]["ymin"] == "auto":
        ymin = 0.0
    else:
        ymin = float(axes["y-axe"]["ymin"])
    if axes["y-axe"]["ymax"] == "auto":
        ymax = 0.0
        ysearch= True 
    else:
        ymax = float(axes["y-axe"]["ymax"])
        ysearch = False
        
    #create figure
    pylab.figure(1,(length,width))
    pylab.subplots_adjust(left,bottom,right,top)
    pylab.ticklabel_format(style = "sci", scilimits = (-2,5))
    
    legend_labels = []
    #get cost_critera
    cost_criteria = None
    for file in filenames:
        try:
            legend_labels.append(information[file][legend_parameter])
        except:
            try: 
                legend_labels.append(information[file][legend_parameter.replace(" ", "_")])
            except:
                for key in information[file]["cost_criteria"]:
                    try:
                        legend_labels.append(information[file]["cost_criteria"][key][legend_parameter])
                    except:
                        try:
                            legend_labels.append(information[file]["cost_criteria"][key][legend_parameter.replace(" ","_")])
                        except:
                            pass
        if cost_criteria == None: 
            criteria = information[file]["cost_criteria"].keys()
            for crit in criteria:
                if crit in file:
                    cost_criteria = crit
        elif cost_criteria in file:
            continue
        else:
            raise IndexError
    
    number_of_files = len(filenames)
    """ determine colors for the different data structures """
    ds_colors = [rgb2hex(hsv_to_rgb((x+1)/float(number_of_files), 0.7, 0.5)) for x in range(number_of_files)]
    avg_mus = {}
    avg_sigmas = {}
    for file in filenames:
        dict_of_crit = information[file]["cost_criteria"][cost_criteria]
        mean_value = float(dict_of_crit["mean_value"])
        avg_mus.update({file : mean_value})
        sigma = float(dict_of_crit["standard_deviation"])
        avg_sigmas.update({file : sigma})
        
    """ determine plot range """
    if xsearch:
        for file in filenames:
            xmin = min(xmin, avg_mus[file] - 3 * avg_sigmas[file])
            xmax = max(xmax, avg_mus[file] + 5 * avg_sigmas[file])
    stepsize = (xmax - xmin) / float(num_bins)
    calculated_bins = pylab.frange(xmin, xmax, stepsize)    
    """ plot entries """
    i = 0
    for file in filenames:
        data = numpy.load(path.join(FOLDER_FOR_DATA, file))
        """ normalize """
        histo, bins = histogram(data, calculated_bins)
        histo = numpy.array(histo, dtype=numpy.float)
        histo *= 1.0/float(information[file]["samples"])
        if numpy.amax(histo) > ymax and ysearch:
            ymax = numpy.amax(histo)
        bincenters = 0.5*(bins[1:] + bins[:-1])
        if len(legend_labels) >1:
            pylab.plot(bincenters, histo, ',', alpha=0.7, color=ds_colors[i], label=legend_labels[i])
        else:
            pylab.plot(bincenters, histo, ',', alpha=0.7, color=ds_colors[i])
        i += 1
    #set axis names    
    pylab.xlabel(xlabel)
    pylab.ylabel(ylabel)
    #set axis limits
    pylab.axis([xmin, xmax, ymin, ymax])
    #set grid
    pylab.grid(True)
    #set title
    pylab.suptitle(title)
    #create legend --> color map
    if len(legend_labels) >1:
        pylab.legend(title = legend_title, loc = legend_position)
    #pylab.axis('tight')
    if create_png:
        i = 1
        filename = 'Cost Distribution %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        while path.exists(FOLDER_FOR_PLOTS + '/' + filename):
            i += 1
            filename = 'Cost Distribution %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        pylab.savefig(FOLDER_FOR_PLOTS + '/' + filename)
        pylab.close()
        return filename
    else:
        pylab.show()
        
if __name__ == '__main__':
    test_entries = [{'benchmark' : 'xerox.yal',
                     'data structure' : '3D Slicing Tree',
                     'filename' : 'lalelu.npy',
                     'samples' : 2000000,
                     'depth' : None,
                     'cost criteria' : 'MPMWE',
                     'mean value' : None,
                     'standard deviation' : None},
                     {'benchmark' : 'xerox.yal',
                     'data structure' : 'Sequence Triple',
                     'filename' : 'lilalu.npy',
                     'samples' : 2000000,
                     'depth' : None,
                     'cost criteria' : 'MPMWE',
                     'mean value' : None,
                     'standard deviation' : None},
                     {'benchmark' : 'xerox.yal',
                     'data structure' : 'T-Tree',
                     'filename' : 'Monte Carlo 2010-07-07 142112 01 - MPMWE TTree xerox 2000000.npy',
                     'samples' : 2000000,
                     'depth' : None,
                     'cost criteria' : 'MPMWE',
                     'mean value' : None,
                     'standard deviation' : None},
                     {'benchmark' : 'xerox.yal',
                     'data structure' : 'Sequence Quintuple',
                     'filename' : 'lulila.npy',
                     'samples' : 2000000,
                     'depth' : None,
                     'cost criteria' : 'MPMWE',
                     'mean value' : None,
                     'standard deviation' : None}]
    
    test_hentry = {'benchmark' : 'xerox.yal',
                   'data structure' : 'T-Tree',
                   'filename' : 'Monte Carlo 2010-07-07 142112 01 - MPMWE TTree xerox 2000000.npy',
                   'samples' : 2000000,
                   'cost criteria' : 'MPMWE'}
    
    test_ventry = {'benchmark' : 'xerox.yal',
                   'data structure' : 'T-Tree',
                   'filename' : 'Monte Carlo 2010-07-07 142112 02 - HESA TTree xerox 2000000.npy',
                   'samples' : 2000000,
                   'cost criteria' : 'HESA'}
    plot_cost_distribution(test_entries)
    