'''implements function to plot the serached solution space of two correlated data sets by a optimization algorithm

This module implements function to plot the serached solution space of two correlated data sets by a optimization algorithm.
It is used to show the area where possible solutions are found. Combined with a equal plot of a Monte Carlo search
of the same data strucutre - benchmark combination, it can show the area where the algorithm was searching for a better solution.
The function allows the use of the created iteration files of the algorithms so the different iteration indices are separated by colors.

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
'''


import numpy
from os import path
from time import strftime
import matplotlib.pyplot as pylab

from dsc_suite.analyses.standards import COLORS
from dsc_suite.tools.toolpaths import FOLDER_FOR_DATA, FOLDER_FOR_PLOTS

def plot_2D_scatter_plot(filenames, information, parameter, create_png=False):
    '''Plots a scatter plot of two cost criteria.
    
    Plots a scatter plot of two cost criteria. Useful to see the correlation between the different solutions.
    With a third file with the iteration information (ITER) a color map is created to show the development depending on the iteration index.
    
    @param filenames: filenames relative to the default folder of data
    @type filenames: list
    @param information: information about the files in filenames extracted from XML data base
    @type information: dictionary
    @param parameter: parameters for the diagram layout
    @type parameter: dictionary
    @param create_png: create a png file(True) from diagram or show interactive mode(False)
    @type create_png: boolean
    '''
    if len(filenames) != 2 or len(filenames) != 3:
        #raise Error
        pass
    
    #get different dictionaries
    labels = parameter["labels"]
    sizes = parameter["sizes"]
    axes = parameter["axes"]
    
    #get information about lables
    title = labels["title"]
    legend_position = labels["legend-position"]
    legend_title = labels["legend-title"]
    #legend_parameter = labels["legend-parameter"]
    xlabel = labels["x-label"]
    ylabel = labels["y-label"]
    
    #get information about size
    width = sizes["dimensions"]["width"]
    length = sizes["dimensions"]["length"]
    left = sizes["margins"]["left"]
    right = sizes["margins"]["right"]
    top = sizes["margins"]["top"]
    bottom = sizes["margins"]["bottom"]
    
    print axes
    #get information about axes
    if axes["x-axe"]["xmin"] == "auto":
        xmin = 0.0
    else:
        xmin = float(axes["x-axe"]["xmin"])
    if axes["x-axe"]["xmax"] == "auto":
        xsearch = True
        xmax = 0.0
    else:
        xmax = float(axes["x-axe"]["xmax"])
        xsearch = False
    if axes["y-axe"]["ymin"] == "auto":
        ymin = 0.0
    else:
        ymin = float(axes["y-axe"]["ymin"])
    if axes["y-axe"]["ymax"] == "auto":
        ymax = 0.0
        ysearch= True 
    else:
        ymax = float(axes["y-axe"]["ymax"])
        ysearch = False
  
    #create figure
    pylab.figure(1,(length,width))
    pylab.subplots_adjust(left,bottom,right,top)
    pylab.ticklabel_format(style = "sci", scilimits = (-2,5))
    color_array = None
    print_dict = {}
    norm_values = []
    criteria = []
    criteria_sort = []
    print_array = []
    
    for file in filenames:
        dictionary = information[file]["cost_criteria"]
        for key in dictionary.keys():
            if str(dictionary[key]["filename"]) == str(file):
                criteria.append(str(key))
                print_dict.update({key : numpy.load(path.join(FOLDER_FOR_DATA, file))})
    for crit in criteria:
        sum_dict = information[file]["cost_criteria"]["SUM"]
        for key in sum_dict:
            if crit in key and "norm" in key:
                norm_values.append(float(sum_dict[key]))
                criteria_sort.append(crit)
                print_array.append(print_dict[crit])
    
    for i in range(0,len(print_array),1):
        print_array[i] = print_array[i]/norm_values[i]
        if i == 0: 
            if xmax < numpy.amax(print_array[i]) and xsearch:
                xmax = numpy.amax(print_array[i])*1.01
        elif i == 1:
            if ymax < numpy.amax(print_array[i]) and ysearch:
                ymax = numpy.amax(print_array[i])*1.01
        else:
            return
    for crit in criteria_sort:
        criteria.remove(crit)
    if len(criteria) > 1:
        return
    for crit in criteria:
        criteria_sort.append(crit)
        color_array = print_dict[crit]
    
    #create different color steps
    if len(filenames) == 3:
        uniques = list(numpy.unique(color_array))
        needed_colors = len(uniques)
        quot = int(needed_colors/len(COLORS))+1
        sep_uni = []
        for i in range(0,needed_colors,quot):
            sep_uni.append(uniques[i:(i+quot)])
            
        for values in sep_uni:
            xarray = []
            yarray = []
            color_code = COLORS[sep_uni.index(values)]
            lab = ""
            for uni in values:
                if values.index(uni) == 0:
                    lab += str(round(uni,2))
                elif values.index(uni) == len(values)-1:
                    lab += " - " + str(round(uni,2))
                for i in range(0,len(color_array),1):
                    if uni == color_array[i]:
                        xarray.append(print_array[0][i])
                        yarray.append(print_array[1][i])
            pylab.plot(xarray,yarray,"o", color=color_code, label = lab)
            
    if len(filenames) == 2:     
        pylab.plot(print_array[0],print_array[1],",")
    #set axis names    
    pylab.xlabel(xlabel)
    pylab.ylabel(ylabel)
    #set axis limits
    pylab.axis([xmin, xmax, ymin, ymax])
    #set grid
    pylab.grid(True)
    #set title
    pylab.suptitle(title)
    #create legend --> color map
    if len(filenames) == 3:
        pylab.legend(title = legend_title, loc = legend_position)
    #create png
    if create_png:
        i = 1
        filename = '2D scatter plot %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        while path.exists(FOLDER_FOR_PLOTS + '/' + filename):
            i += 1
            filename = '2D scatter plot %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        pylab.savefig(FOLDER_FOR_PLOTS + '/' + filename)
        pylab.close()
        return filename
    else:
        pylab.show()