"""implements the basic data structure interface.

This module implements the basic class of the data structures.
All used data structures inherit from class.

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>

@var COST_CRITERIA_LIST: dictionary with the names of the supported cost evaluation criteria
    - key: name of the criteria (short form)
    - value: list with information
        - 0: index of the criteria
        - 1: long term of criteria
        - 2: description of criteria
@type COST_CRITERIA_LIST:
"""

from dsc_suite.routing.estimations import middle_point_manhatten
from random import randrange

"""dictionary of supported cost evaluation methods
Order is important as used in the cost_evaluation function,
as a dictionary don't represent an ordered list, an index is
required.
- key: name of the criteria (short form)
    - value: list with information
        - 0: index of the criteria
        - 1: long term of criteria
        - 2: description of criteria

"""
COST_CRITERIA_LIST = {'MPMWE' : [0, 'Middle point Manhattan wire-length estimation', 'Calculates shortest Manhattan distance between two points. Assumes the Pins of each module in its middle. Supports net weights (e.g., as in the YAL benchmarks).'],
                      'HESA' : [1, 'Half of enveloping surface area', 'Calculates the half of the surrounding (bounding) box area, thus includes unused whitespace.']}


##### DATA STRUCTURE CLASS #####################################################
class DataStructure(object):
    """Defines basic data structure interface.
    Used by Optimization Algorithms to access the implemented data
    structure subclasses. The needed module data is obtained using the
    Benchmark class. The most important variable is data, which stores
    the benchmark information. It's format is as following:
    data = {modulename : [moduletype, dimensions, connections]}
    where dimensions is a list and connections is a dictionary.
    
    @cvar MODULETYPE: index of the module type
    @type MODULETYPE: int
    
    @cvar DIMENSIONS: index of the dimensions
    @type DIMENSIONS: int
    
    @cvar CONNECTIONS: index of the connections
    @type CONNECTIONS: int
    
    @cvar ORIGINAL: index of the original version
    @type ORIGINAL: int
    
    @cvar ROTATED: index of the rotated version
    @type ROTATED: int
    
    @cvar name: name of data structure
    @type name: string
    """
    # static members / constants
    MODULETYPE = 0
    DIMENSIONS = 1
    CONNECTIONS = 2

    ORIGINAL = 0
    ROTATED = 1

    name = 'Data Structure'

    def __init__(self, benchmark):
        """Creates a new instance.
        
        @param benchmark: A benchmark instance of a loaded benchmark.
        @type Benchmark
        """
        self.benchmark = benchmark
        # the following attributes are needed and concretized
        # in the according subclasses
        self.num_parts = 0
        self.part_lengths = []
        self.num_representations = 0
        self.best_representation = None
        #self.best_representation_index = None
        self.worst_solution = None
        # cost 1 ... footprint area
        # cost 2 ... total wirelength
        # single parts dont belong necessarily to the same solution
        self.worst_cost_parts = [0, 0]
        self.best_cost_parts = [float('inf'), float('inf')]
        self.cost_part_names = ['Half Surface', 'Wirelength']
        
    @staticmethod
    def __abstract():
        """Static helper method to implement abstract methods """
        import inspect
        caller = inspect.getouterframes(inspect.currentframe())[1][3]
        raise NotImplementedError(caller + ' must be imnplemented in subclass')
    
    def set_benchmark(self, benchmark):
        """Sets the currently used benchmark.
        Loading benchmark into data structure. So all the important informations
        (e.g., number of modules, dimensions, ...) can be used.
        
        @param benchmark: A benchmark instance of a loaded benchmark.
        @type benchmark: Benchmarks
        """
        # TODO: Really required?? If yes, update data structure relevant
        #       information (e.g., part_lengths, num_representations)!!!
        self.__abstract()
       
    def get_all_representations(self):
        """Generates all possible abstract solutions.
        This method generates all possible abstract solutions. The returned
        format is strongly dependend on the underlying data structure. When
        benchmarks with a big number of modules is loaded very long
        computation and intense memory and processor usage can occur!
        
        @return: Returns a list of all possible abstract solutions.
        @rtype: list
        """
        representations = []
        i = 0
        while i < self.num_representations:
            representations.append(self.get_representation(i))
            i += 1
        return representations

    def get_part_of_representation(self, part_index, part_number):
        """Gets a single element out of a part of the abstract representation.
        part_number between 0 and part_lengths[part_index]
        
        @param part_index:
        @type part_index: int
        
        @param part_number:
        @type part_number: int
        """
        self.__abstract()

    def get_representation(self, number):
        """Gets a specific representation.
        number ranges between 0 and num_representations-1
        This is a global mapping method to get a specific representation out
        of all the abstract representations. Not optimal, cause its used very often to generate
        the solution space. So take care!
        
        @param number: the number of the representation to return
        @type number: int
        
        @return: Returns the representation at the index number out of all possible representations
        @rtype: list
        """
        # this function is called very often while generating
        # the whole solution space, so the import function or
        # a lambda function at this point could be sub optimal
        assert(number < self.num_representations)
        representation = []
        for part_index in range(self.num_parts):
            part_number = number//reduce(lambda a,b:a*b, self.part_lengths[part_index+1:]+[1]) % self.part_lengths[part_index]
            representation.append(self.get_part_of_representation(part_index, part_number))
        return representation

    def get_part_numbers(self, number):
        """Gets the part numbers.
        
        @param number:
        @type number: int
        
        @return:
        @rtype: list 
        """
        assert(number < self.num_representations)
        part_numbers = []
        for part_index in range(self.num_parts):
            part_number = number//reduce(lambda a,b:a*b, self.part_lengths[part_index+1:]+[1]) % self.part_lengths[part_index]
            part_numbers.append(part_number)
        return part_numbers

    def convert_part_numbers(self, part_numbers):
        """Converts the part numbers.
        
        @param part_numbers:
        @type part_numbers:
        
        @return:
        @rtype:
        """
        number = 0
        for part_index in range(self.num_parts):
            number += part_numbers[part_index] * reduce(lambda a,b:a*b, self.part_lengths[part_index+1:]+[1])
        return number

    def __getitem__(self, key):
        """Gets a specific representation.
        Does same as get_representation()
        
        @param key: the number of the representation to return
        @type key: int
        
        @return: Returns the representation at the index number out of all possible representations
        @rtype: list
        """
        return self.get_representation(key)
    
    # TODO: def get_number_of_representation ???

    def get_random_representation(self):
        """Gets a random representation.
        This function gets a random representation out of all possible representations.
        
        @return: Returns a random representation out of all possible representations.
        @rtype: list
        """
        repr_number = randrange(self.num_representations)
        return self.get_representation(repr_number)
        
        
    def generate_solution_space(self):
        """OBSOLETE!!!
        assuming seperate lists, sequences, etc.
        (e.g., representations = (locii1_permutations, locii2_permutations))
        so that a for each element in each part of the datastructrure
        the combination which each other element combination of the other parts
        is assumed to be the abstract_solution.
        This is done due to lower memory usage.
        """
        print "Generating solution space"
        print "Number of solutions: ", self.num_representations
        counter = 0
        costs_list = []
        while counter < self.num_representations:
            costs_list.append(self.cost_evaluation(self.get_representation(counter)))
            counter += 1
        return costs_list
        
    def create_packing(self, representation):
        """Returns packing in dictionary format.
        packing = {'name' : [x, y, z, w, h, d], ...}
        
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the created packing.
        @rtype: dictionary
        """
        self.__abstract()

    def packing_to_representation(self, packing):
        """Creates a representation out of a packing.
        
        @param packing: a packing of a current representation
        @type packing: dictionary
        
        @note: is not possible with every data structure
        """
        self.__abstract()

    def cost_evaluation(self, representation, criteria=['MPMWE', 'HESA'], return_packing=False):
        """Calculates and returns costs of current representation applied to benchmark.
        This pre-implemented cost_evaluation can be used data structure
        independent. But for some specific data structures it could be
        possible to optimize the evaluation by using internal information.
        
        Supported criteria listed in COST_CRITERIA_LIST.
        Short names are used. Parameter criteria is in list format.
        
        !!Instead of changing calculation of existent criteria, add a new one!!
        
        Weights and cost factors have to be implemented inside optimization. 
        
        @param representation: a representation list
        @type representation: list
        
        @param criteria: cost evaluation criteria to calculate
        @type criteria: list
        
        @param return_packing: Sets the return behavior.
            - True: Enabeld the extra return of the packing.
            - False: Disables the extra return of the packing.
        @type return_packing: boolean
        
        @return: Returns a list with the calculated values for the criteria and if enabled the packing.
        @rtype: list, (dictionary)
        """
        packing = self.create_packing(representation)
        # TODO check whether or not packing is overlapping free
        # if it is done it could be runtime expensive
        # from dsc_suite.tools.geometry import valid_packing
        # assert(valid_packing(packing))
        if 'HESA' in criteria:
            nod = self.benchmark.status['number of dimensions']
            temp = max(packing.values(), key=lambda x: x[0]+x[nod])
            width = temp[0] + temp[nod]
            temp = max(packing.values(), key=lambda x: x[1]+x[1+nod])
            height = temp[1] + temp[1+nod]
            temp = max(packing.values(), key=lambda x: x[2]+x[2+nod])
            depth = temp[2] + temp[2+nod]
            cost_HESA = width * height + width * depth + height * depth
        else:
            cost_HESA = None
        if 'MPMWE' in criteria:
            cost_MPMWE = 0.0
            for i in packing:
                temp = self.benchmark.connections[i]
                for j in temp:
                    net_weight = self.benchmark.connections[i][j]
                    net_length = middle_point_manhatten(packing[i], packing[j])
                    cost_MPMWE += net_weight*net_length
        else:
            cost_MPMWE = None
        """ Attend to conformity with COST_CRITERIA_LIST """
        costs = [cost_MPMWE, cost_HESA]
        if return_packing:
            return [costs[i] for i in [COST_CRITERIA_LIST[c][0] for c in criteria]], packing            
        return [costs[i] for i in [COST_CRITERIA_LIST[c][0] for c in criteria]]            

    def get_operations(self):
        """Returns a dictionary of possible operations onto a abstract solution.
        The higher the globality_factor, the bigger is the solution change
        This factor is given by the authors but it also can be determined
        in test runs evaluation the cost influences caused by applying the
        according function.
        
        @return: Returns a dictionary of possible operations onto a abstract solution.
        structure: {'name' : [function, globality_factor]}
        @rtype: dictionary
        """
        self.__abstract()

    def merge_representations(self, representation_a, representation_b):
        """Merges two representations into all possible children.
        Typically, used for genetic and/or evolutionary algorithms.
        Returns a list with all possible children.
        
        @param representation_a: a representation list
        @type representation_a: list
        
        @param representation_b: a representation list
        @type representation_b: list
        
        @return: Returns a list with all possible representations created by merging the two given.
        @rtype: list
        """
        self.__abstract()
        