"""Implements the 3D Moving Block Sequence (c) Robert Fischbach, 2011

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""
from dsc_suite.ds.data_structure import DataStructure
from dsc_suite.tools.combinatorics import EnumerativeCombinatorics as EC
from random import randrange, randint
from copy import deepcopy


##### T-TREE SUBCLASS ####################################################
class MBS(DataStructure):
    """Implements 3D-MBS layout representation.
    possibility of block-mirroring already implemented in trial_20110104

    mbs format : [module_permutation, insert_positions, module_rotations]
    
    module_permutation = [block_Y, block_Z, block_X, ...]
    insert_positions = ['x', 'y', 'z',
                        'xy', 'xz', 'yx', 'yz', 'zx', 'zy',
                        'xyz', 'xzy', 'yxz', 'yzx', 'zxy', 'zyx']
    module_rotations: [x_rotations, y_rotations, z_rotations]
                       x_rotations = [rot0, rot1, rot2, ...]
                       rotX = integer, multiple of 90 degrees (i.e., 0, 1, 2, 3)    
                      rotation around the given axis
    """
    name = '3D Moving Block Sequence'
    
    insert_position_pool = ['x', 'y', 'z',
                            'xy', 'xz', 'yx', 'yz', 'zx', 'zy',
                            'xyz', 'xzy', 'yxz', 'yzx', 'zxy', 'zyx']
    
    def __init__(self, benchmark):
        """Creates a new instance.
        
        @param benchmark: A benchmark instance of a loaded benchmark.
        @type Benchmark
        """
        assert(benchmark.status['number of dimensions'] == 3)
        DataStructure.__init__(self, benchmark)
        from operator import mul
        self.num_blocks = len(benchmark.blocks)
        self.num_insertions = self.num_blocks - 1
        self.num_parts = 2 + 1
        self.part_lengths = [EC.get_num_permutations(benchmark.blocks),
                             EC.get_num_variations(self.insert_position_pool, self.num_insertions),
                             2**self.num_blocks]
        self.num_representations = reduce(mul, self.part_lengths)
        # transform into mbs initial data 
        self.initial_data = {}
        for module_name in benchmark.blocks:
            self.initial_data[module_name] = {}
            x_dim, y_dim, z_dim = benchmark.dimensions[module_name] 
            self.initial_data[module_name]['dimensions'] = [x_dim, y_dim, z_dim]
            self.initial_data[module_name]['position'] = [0.0, 0.0, 0.0]
            # handle left and right faces, point of reference is X, a is Y, b is Z
            self.initial_data[module_name]['left_faces'] = [{'por' : 0, 'a1' : 0, 'b1' : 0, 'a2' : y_dim, 'b2' : z_dim}]
            self.initial_data[module_name]['right_faces'] = [{'por' : x_dim, 'a1' : 0, 'b1' : 0, 'a2' : y_dim, 'b2' : z_dim}]
            # handle bottom and top faces, point of reference is Y, a is X, b is Z
            self.initial_data[module_name]['bottom_faces'] = [{'por' : 0, 'a1' : 0, 'b1' : 0, 'a2' : x_dim, 'b2' : z_dim}]
            self.initial_data[module_name]['top_faces'] = [{'por' : y_dim, 'a1' : 0, 'b1' : 0, 'a2' : x_dim, 'b2' : z_dim}]
            # handle front and back faces, point of reference is Z, a is X, b is Y
            self.initial_data[module_name]['front_faces'] = [{'por' : 0, 'a1' : 0, 'b1' : 0, 'a2' : x_dim, 'b2' : y_dim}]
            self.initial_data[module_name]['back_faces'] = [{'por' : z_dim, 'a1' : 0, 'b1' : 0, 'a2' : x_dim, 'b2' : y_dim}]

    def __mbs_translate(self, module_name, x, y, z, module_data):
        """
        
        @param module_name: name of module
        @type module_name: string
        
        @param x:
        @type x: float
        
        @param y:
        @type y: float
        
        @param z:
        @type z: float
        
        @param module_data: geometric data of all modules
        @type module_data: dict
        """
        module_data[module_name]['position'][0] += x
        module_data[module_name]['position'][1] += y
        module_data[module_name]['position'][2] += z
        # handle left and right faces, point of reference is X, a is Y, b is Z
        for face in module_data[module_name]['left_faces'] + module_data[module_name]['right_faces']:
            face['por'] += x
            face['a1'] += y 
            face['a2'] += y 
            face['b1'] += z 
            face['b2'] += z 
        # handle bottom and top faces, point of reference is Y, a is X, b is Z
        for face in module_data[module_name]['bottom_faces'] + module_data[module_name]['top_faces']:
            face['por'] += y
            face['a1'] += x 
            face['a2'] += x 
            face['b1'] += z 
            face['b2'] += z 
        # handle front and back faces, point of reference is Z, a is X, b is Y
        for face in module_data[module_name]['front_faces'] + module_data[module_name]['back_faces']:
            face['por'] += z
            face['a1'] += x 
            face['a2'] += x 
            face['b1'] += y 
            face['b2'] += y 


    def __mbs_rotate(self, module_name, axis, angle, module_data):
        """Rotate module around axis by given angle. Rotation around origin (local 0/0/0).
        
        @note:  angle = i -> i*90 -> i*pi/2
        
        module_name ... name of module to rotate
        module_data ... geometric data of all modules (dict)
        axis ... axis around which to rotate (x, y, z)
        angle ... only integer are allowed, multiples of 90 degrees
        
        @param module_name: name of module to rotate
        @type module_name: string
                
        @param axis: axis around which to rotate (x,y,z)
        @type axis: char
        
        @param angle: angle of rotation, multiples of 90 degree
        @type angle: int
        
        @param module_data: geometric data of all modules
        @type module_data: dict
        """
        assert module_data[module_name]['position'] == [0, 0, 0]
        assert type(angle) == int
        assert axis == 'x' or axis == 'y' or axis == 'z'

        cos_lut = {0 : 1, 1 : 0, 2 : -1, 3 : 0}
        sin_lut = {0 : 0, 1 : 1, 2 : 0, 3 : -1} 
        angle = angle % 4
        if angle == 0:
            return
        # rotate by right hand rule, retype faces
        if axis == 'x':
            old_faces = ['back_faces', 'bottom_faces', 'front_faces', 'top_faces']
            new_faces = old_faces[angle:] + old_faces[:angle]
            old_faces += ['left_faces', 'right_faces']
            new_faces += ['left_faces', 'right_faces']
        elif axis == 'y':
            old_faces = ['left_faces', 'back_faces', 'right_faces', 'front_faces']
            new_faces = old_faces[angle:] + old_faces[:angle]
            old_faces += ['bottom_faces', 'top_faces']
            new_faces += ['bottom_faces', 'top_faces']
        elif axis == 'z':
            old_faces = ['left_faces', 'bottom_faces', 'right_faces', 'top_faces']
            new_faces = old_faces[angle:] + old_faces[:angle]
            old_faces += ['front_faces', 'back_faces']
            new_faces += ['front_faces', 'back_faces']
        module_data[module_name][new_faces[0]], \
        module_data[module_name][new_faces[1]], \
        module_data[module_name][new_faces[2]], \
        module_data[module_name][new_faces[3]] = \
        module_data[module_name][old_faces[0]], \
        module_data[module_name][old_faces[1]], \
        module_data[module_name][old_faces[2]], \
        module_data[module_name][old_faces[3]]
        for i in range(6):
            for face in module_data[module_name][new_faces[i]]:
                # transformation from por, a and b into x, y, z, depending on old face type
                if old_faces[i] == 'left_faces' or old_faces[i] == 'right_faces':
                    # handle left and right faces, por is X, a is Y, b is Z
                    x1_old = x2_old = face['por']
                    y1_old = face['a1'] 
                    y2_old = face['a2'] 
                    z1_old = face['b1'] 
                    z2_old = face['b2'] 
                elif old_faces[i] == 'bottom_faces' or old_faces[i] == 'top_faces':
                    # handle bottom and top faces, por is Y, a is X, b is Z
                    y1_old = y2_old = face['por']
                    x1_old = face['a1'] 
                    x2_old = face['a2'] 
                    z1_old = face['b1'] 
                    z2_old = face['b2']
                elif old_faces[i] == 'front_faces' or old_faces[i] == 'back_faces':
                    # handle front and back faces, por is Z, a is X, b is Y
                    z1_old = z2_old = face['por']
                    x1_old = face['a1'] 
                    x2_old = face['a2'] 
                    y1_old = face['b1'] 
                    y2_old = face['b2']
                else:
                    raise ValueError('impossible')
                # rotation of coordinates, depending on axis and angle
                if axis == 'x':
                    y1_new = y1_old * cos_lut[angle] - z1_old * sin_lut[angle]
                    y2_new = y2_old * cos_lut[angle] - z2_old * sin_lut[angle]
                    z1_new = y1_old * sin_lut[angle] + z1_old * cos_lut[angle]
                    z2_new = y2_old * sin_lut[angle] + z2_old * cos_lut[angle]
                    x1_new = x1_old
                    x2_new = x2_old
                elif axis == 'y':
                    x1_new = x1_old * cos_lut[angle] + z1_old * sin_lut[angle]
                    x2_new = x2_old * cos_lut[angle] + z2_old * sin_lut[angle]
                    z1_new = z1_old * cos_lut[angle] - x1_old * sin_lut[angle] 
                    z2_new = z2_old * cos_lut[angle] - x2_old * sin_lut[angle] 
                    y1_new = y1_old
                    y2_new = y2_old
                elif axis == 'z':
                    x1_new = x1_old * cos_lut[angle] - y1_old * sin_lut[angle]
                    x2_new = x2_old * cos_lut[angle] - y2_old * sin_lut[angle]
                    y1_new = x1_old * sin_lut[angle] + y1_old * cos_lut[angle]
                    y2_new = x2_old * sin_lut[angle] + y2_old * cos_lut[angle]
                    z1_new = z1_old
                    z2_new = z2_old
                # back transformation from x, y, z into por, a and b, depending on new face type
                if new_faces[i] == 'left_faces' or new_faces[i] == 'right_faces':
                    # handle left and right faces, por is X, a is Y, b is Z
                    assert x1_new == x2_new
                    face['por'] = x1_new
                    face['a1'] = min(y1_new, y2_new)
                    face['a2'] = max(y1_new, y2_new)
                    face['b1'] = min(z1_new, z2_new)
                    face['b2'] = max(z1_new, z2_new)
                elif new_faces[i] == 'bottom_faces' or new_faces[i] == 'top_faces':
                    # handle bottom and top faces, por is Y, a is X, b is Z
                    assert y1_new == y2_new
                    face['por'] = y1_new
                    face['a1'] = min(x1_new, x2_new)
                    face['a2'] = max(x1_new, x2_new)
                    face['b1'] = min(z1_new, z2_new)
                    face['b2'] = max(z1_new, z2_new)
                elif new_faces[i] == 'front_faces' or new_faces[i] == 'back_faces':
                    # handle front and back faces, por is Z, a is X, b is Y
                    assert z1_new == z2_new
                    face['por'] = z1_new
                    face['a1'] = min(x1_new, x2_new)
                    face['a2'] = max(x1_new, x2_new)
                    face['b1'] = min(y1_new, y2_new)
                    face['b2'] = max(y1_new, y2_new)
                else:
                    raise ValueError('impossible')
        # rotation of bounding box, depending on axis and angle
        x_old, y_old, z_old = module_data[module_name]['dimensions']
        if axis == 'x':
            y_new = y_old * cos_lut[angle] - z_old * sin_lut[angle]
            z_new = y_old * sin_lut[angle] + z_old * cos_lut[angle]
            x_new = x_old
        elif axis == 'y':
            x_new = x_old * cos_lut[angle] + z_old * sin_lut[angle]
            z_new = z_old * cos_lut[angle] - x_old * sin_lut[angle]
            y_new = y_old 
        elif axis == 'z':
            x_new = x_old * cos_lut[angle] - y_old * sin_lut[angle]
            y_new = x_old * sin_lut[angle] + y_old * cos_lut[angle]
            z_new = z_old
        module_data[module_name]['dimensions'] = [abs(x_new), abs(y_new), abs(z_new)]
        # translate back to 0/0/0
        if x_new >= 0:
            x_new = 0
        if y_new >= 0:
            y_new = 0
        if z_new >= 0:
            z_new = 0
        self.__mbs_translate(module_name, abs(x_new), abs(y_new), abs(z_new), module_data)
        module_data[module_name]['position'] = [0, 0, 0]

    def __mbs_find_overlap(self, module_name, direction, placed_modules, module_data):
        """
        
        @param module_name: name of module to rotate
        @type module_name: string
                
        @param direction: 
        @type direction: char
        
        @param placed_modules:
        @type placed_modules:
        
        @param module_data: geometric data of all modules
        @type module_data: dict
        """
        contact_faces = []
        if direction.lower() == 'x':
            shift = module_data[module_name]['position'][0]
            module_faces = 'left_faces' 
            candidate_faces = 'right_faces'
        elif direction.lower() == 'y':
            shift = module_data[module_name]['position'][1]
            module_faces = 'bottom_faces'
            candidate_faces = 'top_faces'
        elif direction.lower() == 'z':
            shift = module_data[module_name]['position'][2]
            module_faces = 'front_faces'
            candidate_faces = 'back_faces'
        else:
            raise ValueError(direction + ' is an invalid direction. Use x, y, or z.')
        shift = round(shift, 5)
        #TODO: here comes the terribly inefficient part!!! to optimize!!!
        for candidate in placed_modules:
            for module_face in module_data[module_name][module_faces]:
                for candidate_face in module_data[candidate][candidate_faces]:
                    if ((module_face['a1'] < candidate_face['a2']) and
                        (module_face['a2'] > candidate_face['a1']) and
                        (module_face['b1'] < candidate_face['b2']) and
                        (module_face['b2'] > candidate_face['b1'])):
                        # there was a floating point error where calculation returns -5e-14 instead of 0, this I round to 5 decimal digits
                        new_shift = round(module_face['por'] - candidate_face['por'], 5)
                        # < 0 if modules are "above" inserted module  
                        if 0 <= new_shift < shift:
                            contact_faces = [[module_face, candidate_face]]
                            shift = new_shift
                        elif new_shift == shift:
                            contact_faces.append([module_face, candidate_face])
        return shift, contact_faces

    def __mbs_insertion(self, module_name, insert_position, placed_modules, placed_modules_bounding_dimensions, module_data, log):
        """
        Assuming initial positions of all modules to be 0/0/0!
        valid insert positions: x, y, z, xy, yx, xz, zx, yz, zy, xyz, yzx, zxy
        and they also give insertion priority
        
        @param module_name: name of module to rotate
        @type module_name: string
        
        @param insert_position:
        @type insert_position:
        
        @param placed_modules:
        @type placed_modules:
        
        @param placed_modules_bounding_dimensions:
        @type placed_modules_bounding_dimensions: 
        
        @param module_data: geometric data of all modules
        @type module_data: dict
        
        @param log:
        @type log:
        """
        if placed_modules == []:
            log.append([module_name, 'INIT', 0, 0, 0])
        else:
            # coordinate to index help dictionary
            c2i = {'x' : 0, 'y' : 1, 'z' : 2}
            # reference coordinate and secondary coordinate to a/b
            rc2ab = {'x' : {'y' : 'a', 'z' : 'b'},
                     'y' : {'x' : 'a', 'z' : 'b'},
                     'z' : {'x' : 'a', 'y' : 'b'}}
            # initial translation
            initial_translation = [0, 0, 0]
            # first coordinate: module_position = dimension_of_bounding_box_of_placed_modules
            i = c2i[insert_position[0]]
            initial_translation[i] = placed_modules_bounding_dimensions[i] 
            # second and third coordinate: module_position = dimension_of_bb - module_dimension
            for pos in insert_position[1:]:
                i = c2i[pos]
                if module_data[module_name]['dimensions'][i] < placed_modules_bounding_dimensions[i]:
                    initial_translation[i] = placed_modules_bounding_dimensions[i] - module_data[module_name]['dimensions'][i]
                else:
                    initial_translation[i] = 0
            x, y, z = initial_translation
            self.__mbs_translate(module_name, x, y, z, module_data)
            log.append([module_name, 'INIT', x, y, z])
            # insertion process
            retry = True
            while retry:
                # high priority move
                shift, contact_faces = self.__mbs_find_overlap(module_name, insert_position[0], placed_modules, module_data)
                if shift > 0:
                    translation = [0, 0, 0]
                    translation[c2i[insert_position[0]]] = -shift
                    x, y, z = translation
                    """
                    would be more effective to combine all translations into a single step,
                    but currently I want to see step by step what is happening
                    """
                    self.__mbs_translate(module_name, x, y, z, module_data)
                    log.append([module_name, 'TRANSLATION', x, y, z])
                retry = False
                # low priority move(s)
                for pos in insert_position[1:]:
                    if not retry:
                        i = c2i[pos]
                        # shift maximum to next barrier
                        limit = self.__mbs_find_overlap(module_name, pos, placed_modules, module_data)[0]
                        if limit > 0:
                            # If there are no contact faces -> high priority direction reached 0.
                            # Thus, low priority directions can be shifted until limit as well. 
                            if len(contact_faces) == 0:
                                shift = limit
                            else:
                                # Determine largest difference of all contact faces!
                                max_shift = 0
                                for module_face, candidate_face in contact_faces:
                                    ab = rc2ab[insert_position[0]][pos]
                                    max_shift = max(module_face[ab+'2'] - candidate_face[ab+'1'], max_shift)
                                # Enough space to bring module down from contact faces.
                                if limit >= max_shift:
                                    assert(max_shift > 0) # should not happen
                                    shift = max_shift
                                else:
                                    shift = limit
                            retry = True
                            translation = [0, 0, 0]
                            translation[i] = -shift
                            x, y, z = translation
                            self.__mbs_translate(module_name, x, y, z, module_data)        
                            log.append([module_name, 'TRANSLATION', x, y, z])
        # bounding box dimensions of placed modules update
        placed_modules_bounding_dimensions[0] = max(placed_modules_bounding_dimensions[0],
                                                    module_data[module_name]['position'][0] + module_data[module_name]['dimensions'][0])
        placed_modules_bounding_dimensions[1] = max(placed_modules_bounding_dimensions[1],
                                                    module_data[module_name]['position'][1] + module_data[module_name]['dimensions'][1])
        placed_modules_bounding_dimensions[2] = max(placed_modules_bounding_dimensions[2],
                                                    module_data[module_name]['position'][2] + module_data[module_name]['dimensions'][2])
        placed_modules.append(module_name)
        
    def __mbs_set_module_positions_zero(self, module_data):
        """ create a module_data copy with positions set back to zero """
        zeroed_data = deepcopy(module_data)
        for module_name in zeroed_data:
            x, y, z = zeroed_data[module_name]['position']
            self.__mbs_translate(module_name, -x, -y, -z, zeroed_data)
            assert zeroed_data[module_name]['position'] == [0, 0, 0]
        return zeroed_data
        
    def __start_3d_mbs(self, module_permutation, insert_positions, initial_module_data, module_rotations=[None,None,None], bb_constraint=None, max_z=None):
        """consider potential rotations, mirrorings, ... on the copy only AND:
        ATTENTION!!! take care for consistency in module_data (i.e., between position and faces)
                
        @param module_permutation:
        @type module_permutation: 
        
        @param insert_positions:
        @type insert_positions:
        
        @param initial_module_data:
        @type initial_module_data:
        
        @param module_rotations:
        @type module_rotations: 
        
        @param bb_constraint:
        @type bb_constraint:
        
        @param max_z:
        @type max_z:    
        """
        module_data = self.__mbs_set_module_positions_zero(initial_module_data)
        placed_modules_bounding_dimensions = [0, 0, 0]
        placed_modules = []
        log = []
        assert len(module_permutation) == len(insert_positions)+1 == len(module_data)
        i = 0
        while i < len(module_permutation):
            x_rotations, y_rotations, z_rotations = module_rotations
            if not x_rotations == None:
                self.__mbs_rotate(module_permutation[i], 'x', x_rotations[i], module_data)
            if not y_rotations == None:
                self.__mbs_rotate(module_permutation[i], 'y', y_rotations[i], module_data)
            if not z_rotations == None:
                self.__mbs_rotate(module_permutation[i], 'z', z_rotations[i], module_data)
            if i == 0:
                self.__mbs_insertion(module_permutation[i], None, placed_modules,
                                     placed_modules_bounding_dimensions, module_data, log)
            else:
                self.__mbs_insertion(module_permutation[i], insert_positions[i-1], placed_modules,
                                     placed_modules_bounding_dimensions, module_data, log)
            if (not max_z == None) and (placed_modules_bounding_dimensions[2] > max_z):
                return None
            """
            if ((not bb_constraint == None) and
                ((placed_modules_bounding_dimensions[0] > bb_constraint[0]) or
                 (placed_modules_bounding_dimensions[1] > bb_constraint[1]) or
                 (placed_modules_bounding_dimensions[2] > bb_constraint[2]))):
                return None
            """
            i += 1
        # important! that modifications on original mbs do not change returned solution
        mbs = (list(module_permutation), list(insert_positions), list(module_rotations))
        return module_data, log, placed_modules_bounding_dimensions, mbs

    def create_packing(self, representation):
        """Returns packing in dictionary format.
        packing = {'name' : [x, y, z, w, h, d], ...}
        
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the created packing.
        @rtype: dictionary
        """
        module_permutation, insert_positions, module_rotations = representation
        module_data = self.__start_3d_mbs(module_permutation, insert_positions, self.initial_data, [None, None, module_rotations])[0]
        packing = {}
        for module_name in module_data:
            packing[module_name] = module_data[module_name]['position'] + module_data[module_name]['dimensions']
        return packing

    def get_random_representation(self):
        """Gets a random representation.
        This function gets a random representation out of all possible representations.
        
        @return: Returns a random representation out of all possible representations.
        @rtype: list
        """
        # generate random module permutation
        module_permutation = EC.get_permutation(self.benchmark.blocks,
                                                randrange(self.part_lengths[0]))
        # generate random insert positions
        insert_positions = EC.get_variation(self.insert_position_pool,
                                            self.num_insertions,
                                            randrange(self.part_lengths[1]))
        # generate random module_rotations, only around z-axis
        module_rotations = EC.get_variation([self.ORIGINAL, self.ROTATED],
                                            self.num_blocks, 
                                            randrange(self.part_lengths[2]))
        # return in 3d-mbs list format
        return [module_permutation, insert_positions, module_rotations]

    def packing_to_representation(self, packing):
        """Creates a representation out of a packing.
        
        @param packing: a packing of a current representation
        @type packing: dictionary
        
        @note: Not implemented and/or available!
        """
        print "Not implemented and/or available!"

    def get_operations(self):
        """Returns a dictionary of possible operations onto a abstract solution.
        The higher the globality_factor, the bigger is the solution change
        This factor is given by the authors but it also can be determined
        in test runs evaluation the cost influences caused by applying the
        according function.
        
        @return: Returns a dictionary of possible operations onto a abstract solution.
        structure: {'name' : [function, globality_factor]}
        @rtype: dictionary
        """
        return {'Half of insert positions new' : [self.half_of_insert_positions_new, 40],
                'New insert position and rotation' : [self.new_insert_position_and_rotation, 30],
                'Swap two modules' : [self.swap_two_modules, 20],
                'Rotate single block in plane (keep z)': [self.rotate_module, 10]}
        # + evtl. exchange subtrees

    def merge_representations(self, representation_a, representation_b):
        """Merges two representations into all possible children.
        Typically, used for genetic and/or evolutionary algorithms.
        Returns a list with all possible children.
        
        In case of 3D-MBS all possible combinations of the
        module permutations, insert positions and rotations
        are generated. Rotations are currently completely taken from
        representation a or b, respectively. Thus 2^3 = 8 different children
        are possible.
        
        @param representation_a: a representation list
        @type representation_a: list
        
        @param representation_b: a representation list
        @type representation_b: list
        
        @return: Returns a list with all possible representations created by merging the two given.
        @rtype: list
        """
        result = []
        for n in range(2**3):
            child = []
            for i in range(3):
                if n // 2**(2-i):
                    child.append(representation_a[i])
                else:
                    child.append(representation_b[i])
                n = n % 2**(2-i)
            result.append(child)
        return result

    def half_of_insert_positions_new(self, representation):
        """
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the changed representation list.
        @rtype: a representation list
        """
        # generate half of insert positions new
        insert_positions = list(representation[1])
        ip_num = 0
        ip_num_limit = len(insert_positions) // 2
        while ip_num < ip_num_limit:
            ip_num += 1
            pos1 = randrange(len(insert_positions))
            pos2 = randrange(len(self.insert_position_pool))
            insert_positions[pos1] = self.insert_position_pool[pos2]
        return [representation[0], insert_positions, representation[2]]

    def new_insert_position_and_rotation(self, representation):
        """
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the changed representation list.
        @rtype: a representation list
        """
        # generate a new insert positions combined with rotation
        insert_positions = list(representation[1])
        pos1 = randrange(len(insert_positions))
        pos2 = randrange(len(self.insert_position_pool))
        while insert_positions[pos1] == self.insert_position_pool[pos2]:
            pos2 = randrange(len(self.insert_position_pool))
        insert_positions[pos1] = self.insert_position_pool[pos2]
        rotations = list(representation[2])
        index = randint(0, len(rotations)-1)
        if rotations[index] == self.ORIGINAL:
            rotations[index] = self.ROTATED
        else:
            rotations[index] = self.ORIGINAL
        return [representation[0], insert_positions, rotations]

    def rotate_module(self, representation):
        """Rotates a module in a 3D-MBS.
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the changed representation list.
        @rtype: a representation list
        """
        rotations = list(representation[2])
        index = randint(0, len(rotations)-1)
        if rotations[index] == self.ORIGINAL:
            rotations[index] = self.ROTATED
        else:
            rotations[index] = self.ORIGINAL
        return representation[:2]+[rotations]

    def swap_two_modules(self, representation):
        """Swaps two modules in a 3D-MBS.
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the changed representation list.
        @rtype: a representation list
        """
        module_permutation = list(representation[0])
        pos1 = pos2 = randrange(len(module_permutation))
        while pos2 == pos1:
            pos1 = randrange(len(module_permutation))
        module_permutation[pos1], module_permutation[pos2] = module_permutation[pos2], module_permutation[pos1]
        return [module_permutation] + representation[1:]
