"""implements the O-Sequence data structure

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""
from dsc_suite.ds.data_structure import DataStructure
from dsc_suite.tools.combinatorics import EnumerativeCombinatorics
from random import randrange, randint


##### SEQUENCE TRIPLE SUBCLASS #################################################
class OSequence(DataStructure):
    """Implements O-Sequence data structure.
    
    O-Sequence format : [module_permutation, section_variation, wall_combination, block_rotations, o_sequence]
    o_sequence optional, None if not given
        
    TODO: detailed docstring
    """
    name = 'O Sequence'

    def __init__(self, benchmark):
        """Creates a new instance.
        
        @param benchmark: A benchmark instance of a loaded benchmark.
        @type Benchmark
        """
        assert(benchmark.status['number of dimensions'] == 3)
        DataStructure.__init__(self, benchmark)
        #from operator import mul
        from dsc_suite.tools.math import fak
        # maybe better combined in a status dictionary
        self.num_blocks = len(benchmark.blocks)
        self.num_parts = 3 + 1 + 1
        self.part_lengths = [fak(self.num_blocks), 3**(self.num_blocks-1), 'unknown', 2**self.num_blocks, 'unknown']
        self.num_representations = 'unknown' #reduce(mul, self.part_lengths)
        # if constructor call takes to long comment out
        #self.update_balancing()
    

    def get_part_of_representation(self, part_index, part_number):
        """Gets a single element out of a part of the abstract representation.
        part_number between 0 and part_lengths[part_index]
        
        @param part_index:
        @type part_index: int
        
        @param part_number:
        @type part_number: int
        """
        # parameter part_index is not used here because all three
        # sequences are equal, but for other data structures or by adding
        # a rotation sequence, this parameter is useful
        if part_index == 0:
            return EnumerativeCombinatorics.get_permutation(self.benchmark.blocks, part_number)
        elif part_index == 1:
            return EnumerativeCombinatorics.get_variation(['X', 'Y', 'Z'],
                                                          len(self.benchmark.blocks-1), part_number)
        elif part_index == 2:
            return 'unknown'
        elif part_index == 3:
            return EnumerativeCombinatorics.get_variation([self.ORIGINAL, self.ROTATED],
                                                          len(self.benchmark.blocks), part_number)

    def packing_to_representation(self, packing):
        """Creates a representation out of a packing.
        
        @param packing: a packing of a current representation
        @type packing: dictionary
        
        @note: Not implemented and/or available!
        """
        print "Not implemented and/or available!"

    def get_random_representation(self):
        """Gets a random representation.
        This function gets a random representation out of all possible representations.
        Still required to test if random solutions are evenly distributed!
        
        @return: Returns a random representation out of all possible representations.
        @rtype: list
        """
        n = self.num_blocks
        sv = []
        wc = []
        p = []
        contingent = {'X' : [n],
                      'Y' : [n],
                      'Z' : [n]}
        i = n - 1
        while i > 0:
            sv.insert(0, ['X', 'Y', 'Z'][randrange(3)])
            len_cont = len(contingent[sv[0]])
            valid_wc_candidates = []
            for wc_candidate in range(1, len_cont+1):
                good_candidate = True
                for l in range(contingent[sv[0]][-wc_candidate]+1, n + 1):
                    a = True
                    b = False
                    for j in contingent[sv[0]][len_cont-wc_candidate+1:]:
                        if (p[j-i-1][0] != sv[0]) and (l in p[j-i-1][1:]):
                            a = False
                    for j in range(contingent[sv[0]][-wc_candidate], l):
                        if (p[j-i-1][0] == sv[0]) and (l in p[j-i-1][1:]):
                            b = True
                    if not (a or b):
                        good_candidate = False
                if good_candidate:
                    valid_wc_candidates.append(wc_candidate)
            wc.insert(0, valid_wc_candidates[randrange(len(valid_wc_candidates))])
            p.insert(0, [sv[0]] + contingent[sv[0]][-wc[0]:])
            del contingent[sv[0]][-wc[0]:]
            contingent['X'].append(i)
            contingent['Y'].append(i)
            contingent['Z'].append(i)
            i -= 1
        temp = []
        for sd in ['X', 'Y', 'Z']:
            temp.append([sd] + contingent[sd])
        p.insert(0, temp)
        module_permutation = EnumerativeCombinatorics.get_permutation(self.benchmark.blocks, randrange(self.part_lengths[0]))
        block_rotations = EnumerativeCombinatorics.get_variation([self.ORIGINAL, self.ROTATED], len(self.benchmark.blocks), randrange(self.part_lengths[3]))
        # sv ... section_variation
        # wc ... wall_combination
        return [module_permutation, sv, wc, block_rotations, p] 

    def get_P(self, slicing_variation, wall_count):
        """
        @param slicing_variation:
        @type slicing_variation:
        
        @param wall_count:
        @type wall_count:
        
        @return:
        @rtype: list
        """
        P_list = []
        sv, wc = slicing_variation, wall_count
        i = len(sv)
        #P_list.insert(0, i+1)
        contingent = {'X' : [i+1],
                      'Y' : [i+1],
                      'Z' : [i+1]}
        while i > 0:
            if len(contingent[sv[i-1]]) < wc[i-1]:
                return None
            P_list.insert(0, [sv[i-1]] + contingent[sv[i-1]][-wc[i-1]:])
            del contingent[sv[i-1]][-wc[i-1]:]
            #P_list.insert(0, i)
            contingent['X'].append(i)
            contingent['Y'].append(i)
            contingent['Z'].append(i)
            i -= 1
        temp = []
        for sd in ['X', 'Y', 'Z']:
            temp.append([sd] + contingent[sd])
        P_list.insert(0, temp)
        return P_list

    def pair_check(self, P_list):
        """
        @param P_list:
        @type P_list: list
        
        @return:
        @rtype: boolean
        """
        for sd in ['X', 'Y', 'Z']:
            test = filter(lambda x:x[0]==sd, P_list[0])[0][1:]
            i = 1
            if test[-1] == i:
                del test[-1]
            else:
                return False
            while i < len(P_list):
                if P_list[i][0] == sd:
                    # open new pair
                    test += P_list[i][1:] 
                # close pair
                i += 1
                if test[-1] == i:
                    del test[-1]
                else:
                    return False
        return True
                    
    def test_no3(self, P_list):
        """
        @param P_list:
        @type P_list: list
        
        @return:
        @rtype: boolean
        """
        p = P_list
        n = len(p)
        i = 1
        while i < n:
            for l in range(p[i][1]+1, n+1):
                a = True
                b = False
                for j in p[i][2:]:
                    if (p[j][0] != p[i][0]) and (l in p[j][1:]):
                        a = False
                for j in range(p[i][1], l):
                    if (p[j][0] == p[i][0]) and (l in p[j][1:]):
                        b = True
                if not (a or b):
                    return False
            i += 1
        return True

                
    def is_valid(self, representation):
        """Checks if given o-sequence is valid.
        O-Sequence format : [module_permutation, section_variation, wall_combination, block_rotations]
        
        #1: section_variation[i] sequence of exclusively X, Y or Z, length at least 1 -> given
        #2: parenthesis system, ordered pairing (X_k, k), (Y_k, k) and (Z_k, k) for k = 1, 2, ..., n
        #3: previously used walls do not conflict with current ones
        
        @param representation: a representation list
        @type representation: list
        """
        p = self.get_P(representation[1], representation[2])
        return self.pair_check(p) and self.test_no3(p)


    def create_packing(self, representation):
        """Returns packing in dictionary format.
        packing = {'name' : [x, y, z, w, h, d], ...}
        
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the created packing.
        @rtype: dictionary
        """
        p = representation[4]
        if p == None:
            sv, wc = representation[1:3]
            p = self.get_P(sv, wc)
        mn = representation[0]
        rotations = representation[3]
        dims = self.benchmark.dimensions
        widths = dict(zip(dims.keys(), zip(*dims.values())[0]))
        heights = dict(zip(dims.keys(), zip(*dims.values())[1]))
        depths = dict(zip(dims.keys(), zip(*dims.values())[2]))
        for i in range(len(rotations)):
            if rotations[i] == self.ROTATED:
                block = self.benchmark.blocks[i]
                widths[block], heights[block] = heights[block], widths[block]
        n = len(p)
        packing = {}
        packing[mn[0]] = [0, 0, 0,
                          widths[mn[0]],
                          heights[mn[0]],
                          depths[mn[0]]]
        for k in range(n-1, 0, -1):
            mod_k = mn[-k]
            if p[k][0] == 'X':
                x = max([packing[mn[-r]][0]+packing[mn[-r]][3] for r in p[k][1:]])
                y = min([packing[mn[-r]][1] for r in p[k][1:]])
                z = min([packing[mn[-r]][2] for r in p[k][1:]])
                packing[mod_k] = [x, y, z,
                                  widths[mod_k],
                                  heights[mod_k],
                                  depths[mod_k]]
            elif p[k][0] == 'Y':
                x = min([packing[mn[-r]][0] for r in p[k][1:]])
                y = max([packing[mn[-r]][1]+packing[mn[-r]][4] for r in p[k][1:]])
                z = min([packing[mn[-r]][2] for r in p[k][1:]])
                packing[mod_k] = [x, y, z,
                                  widths[mod_k],
                                  heights[mod_k],
                                  depths[mod_k]]
            elif p[k][0] == 'Z':
                x = min([packing[mn[-r]][0] for r in p[k][1:]])
                y = min([packing[mn[-r]][1] for r in p[k][1:]])
                z = max([packing[mn[-r]][2]+packing[mn[-r]][5] for r in p[k][1:]])
                packing[mod_k] = [x, y, z,
                                  widths[mod_k],
                                  heights[mod_k],
                                  depths[mod_k]]
        return packing
