"""implements the Slicing Tree data structure

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""
from dsc_suite.ds.data_structure import DataStructure
from dsc_suite.tools.combinatorics import EnumerativeCombinatorics as EC
from dsc_suite.tools.graphtheory import BinaryTrees as BT
from random import randint, randrange
#from distutils.filelist import stat

# sta element format: [tag, parent, left_child, right_child,
#                      x, y, z, width, height, depth]    
STAE_TAG = 0
STAE_PARENT = 1
STAE_LEFT_CHILD = 2
STAE_RIGHT_CHILD = 3
STAE_X = 4
STAE_Y = 5
STAE_Z = 6
STAE_WIDTH = 7
STAE_HEIGHT = 8
STAE_DEPTH = 9

# nur noch zu verstaendnisszwecken
class SlicingTreeArrayCell():
    # attributes: tag, parent, left_child, right_child,
    #             x, y, z, width, height, depth
       
    def __init__(self, tag, parent, left_child, right_child,
                 x, y, z, width, height, depth):
        self.tag = tag
        self.parent = parent
        self.left_child = left_child
        self.right_child = right_child
        self.x = x
        self.y = y
        self.z = z
        self.width = width
        self.height = height
        self.depth = depth    
        
    def __str__(self):
        if self.left_child:
            return '%s-slice (%s, %s, %s) (%s x %s x %s)' % (self.tag,
                                                             self.parent,
                                                             self.left_child,
                                                             self.right_child,
                                                             self.width,
                                                             self.height,
                                                             self.depth)
        else:     
            return '%s @ (%s, %s, %s) (%s x %s x %s)' % (self.tag, self.x,
                                                         self.y, self.z,
                                                         self.width,
                                                         self.height,
                                                         self.depth)

# nur noch zu verstaendnisszwecken
class SlicingTreeArray():
    __data = list()
    
    def __init__(self, operation_variation, module_permutation,
                 topology, widths, heights, depths):
        #assert(len(operation_variation) + 1 == len(module_permutation))
        #assert(len(operation_variation) + len(module_permutation) == len(topology))
        operation_variation = list(operation_variation)
        module_permutation = list(module_permutation)
        topology_relations = BT.generate_relations(topology)
        i = 0
        while i < len(topology):
            if topology[i] == '1':
                # slicing operation, super module
                self.__data.append(SlicingTreeArrayCell(operation_variation.pop(0),
                                                        topology_relations[i][0],
                                                        topology_relations[i][1],
                                                        topology_relations[i][2],
                                                        None, None, None,
                                                        None, None, None))
            else:
                # leaf cell, module
                module_name = module_permutation.pop(0)
                self.__data.append(SlicingTreeArrayCell(module_name,
                                                        topology_relations[i][0],
                                                        topology_relations[i][1],
                                                        topology_relations[i][2],
                                                        0, 0, 0,
                                                        widths[module_name],
                                                        heights[module_name],
                                                        depths[module_name]))
            i += 1

    def __getitem__(self, i):
        return self.__data[i]
        
    def __len__(self):
        return len(self.__data)

    def __str__(self):
        temp = ''
        i = 0
        while i < len(self.__data):
            temp += '%2i: %s\n' % (i, self.__data[i].__str__())
            i += 1
        return temp
    
    def calc_dimensions(self, u=0, x=0, y=0, z=0):
        self[u].x = x
        self[u].y = y
        self[u].z = z
        if self[u].left_child:
            left_child = self[u].left_child
            right_child = self[u].right_child
        else:
            return # u is a leaf cell
        if self[u].tag == 'X':
            self.calc_dimensions(left_child, x, y, z)
            self.calc_dimensions(right_child, x + self[left_child].width, y, z)
            self[u].width = self[left_child].width + self[right_child].width
            self[u].height = max(self[left_child].height, self[right_child].height)
            self[u].depth = max(self[left_child].depth, self[right_child].depth)
        elif self[u].tag == 'Y':
            self.calc_dimensions(left_child, x, y, z)
            self.calc_dimensions(right_child, x, y + self[left_child].height, z)
            self[u].width = max(self[left_child].width, self[right_child].width)
            self[u].height = self[left_child].height + self[right_child].height
            self[u].depth = max(self[left_child].depth, self[right_child].depth)
        else: # self[u].tag == 'Z'
            self.calc_dimensions(left_child, x, y, z)
            self.calc_dimensions(right_child, x, y, z + self[left_child].depth)
            self[u].width = max(self[left_child].width, self[right_child].width)
            self[u].height = max(self[left_child].height, self[right_child].height)
            self[u].depth = self[left_child].depth + self[right_child].depth
            


##### SLICING TREE SUBCLASS ####################################################
class SlicingTree(DataStructure):
    """Implements Slicing Tree data structure.

    Currently implemented as static array representation
    
    slicing_tree format : [operation_variation, module_permutation, topology, block_rotations]
    
    rotations only 0 and 90 degrees    
    
    TODO: detailed docstring
    """
    name = '3D Slicing Tree'

    def __init__(self, benchmark):
        """Creates a new instance.
        
        @param benchmark: A benchmark instance of a loaded benchmark.
        @type Benchmark
        """
        assert(benchmark.status['number of dimensions'] == 3)
        DataStructure.__init__(self, benchmark)
        from operator import mul
        self.num_blocks = len(benchmark.blocks)
        self.num_slices = self.num_blocks - 1
        self.num_parts = 3 + 1
        self.part_lengths = [EC.get_num_variations(['X', 'Y', 'Z'], self.num_slices),
                             EC.get_num_permutations(benchmark.blocks),
                             BT.get_num_binary_trees(self.num_slices),
                             2**self.num_blocks]
        self.num_representations = reduce(mul, self.part_lengths)
        #self.update_balancing()
        

    def get_part_of_representation(self, part_index, part_number):
        """Gets a single element out of a part of the abstract representation.
        part_number between 0 and part_lengths[part_index]
        
        @param part_index:
        @type part_index: int
        
        @param part_number:
        @type part_number: int
        """
        if part_index == 0:
            return EC.get_variation(['X', 'Y', 'Z'], self.num_slices, part_number)
        elif part_index == 1:
            return EC.get_permutation(self.benchmark.blocks, part_number)
        elif part_index == 2:
            return BT.get_binary_tree(self.num_slices, part_number)
        elif part_index == 3:
            return EC.get_variation([self.ORIGINAL, self.ROTATED],
                                    self.num_blocks, part_number)

    # sta element format: [tag, parent, left_child, right_child,
    #                      x, y, z, width, height, depth]    
    def calc_dimensions(self, sta, u=0, x=0, y=0, z=0):
        """
        @param sta:
        @type sta:
        
        @param u:
        @type u: int
        
        @param x:
        @type x: int
        
        @param y:
        @type y: int
        
        @param z:
        @type z: int
        """      
        sta[u][STAE_X] = x
        sta[u][STAE_Y] = y
        sta[u][STAE_Z] = z
        if sta[u][STAE_LEFT_CHILD]:
            left_child = sta[u][STAE_LEFT_CHILD]
            right_child = sta[u][STAE_RIGHT_CHILD]
        else:
            return # u is a leaf cell
        if sta[u][STAE_TAG] == 'X':
            self.calc_dimensions(sta, left_child, x, y, z)
            self.calc_dimensions(sta, right_child, x + sta[left_child][STAE_WIDTH], y, z)
            sta[u][STAE_WIDTH] = sta[left_child][STAE_WIDTH] + sta[right_child][STAE_WIDTH]
            sta[u][STAE_HEIGHT] = max(sta[left_child][STAE_HEIGHT], sta[right_child][STAE_HEIGHT])
            sta[u][STAE_DEPTH] = max(sta[left_child][STAE_DEPTH], sta[right_child][STAE_DEPTH])
        elif sta[u][STAE_TAG] == 'Y':
            self.calc_dimensions(sta, left_child, x, y, z)
            self.calc_dimensions(sta, right_child, x, y + sta[left_child][STAE_HEIGHT], z)
            sta[u][STAE_WIDTH] = max(sta[left_child][STAE_WIDTH], sta[right_child][STAE_WIDTH])
            sta[u][STAE_HEIGHT] = sta[left_child][STAE_HEIGHT] + sta[right_child][STAE_HEIGHT]
            sta[u][STAE_DEPTH] = max(sta[left_child][STAE_DEPTH], sta[right_child][STAE_DEPTH])
        else: # sta[u].tag == 'Z'
            self.calc_dimensions(sta, left_child, x, y, z)
            self.calc_dimensions(sta, right_child, x, y, z + sta[left_child][STAE_DEPTH])
            sta[u][STAE_WIDTH] = max(sta[left_child][STAE_WIDTH], sta[right_child][STAE_WIDTH])
            sta[u][STAE_HEIGHT] = max(sta[left_child][STAE_HEIGHT], sta[right_child][STAE_HEIGHT])
            sta[u][STAE_DEPTH] = sta[left_child][STAE_DEPTH] + sta[right_child][STAE_DEPTH]

    def create_packing(self, representation):
        """Returns packing in dictionary format.
        packing = {'name' : [x, y, z, w, h, d], ...}
        
        @param representation: a representation list
        @type representation: list
        
        @return: Returns the created packing.
        @rtype: dictionary
        """
        operation_variation = list(representation[0])
        module_permutation = list(representation[1])
        topology = representation[2]
        rotations = representation[3]
        topology_relations = BT.generate_relations(topology)
        dims = self.benchmark.dimensions
        widths = dict(zip(dims.keys(), zip(*dims.values())[0]))
        heights = dict(zip(dims.keys(), zip(*dims.values())[1]))
        depths = dict(zip(dims.keys(), zip(*dims.values())[2]))
        for i in range(len(rotations)):
            if rotations[i] == self.ROTATED:
                block = self.benchmark.blocks[i]
                widths[block], heights[block] = heights[block], widths[block]
        sta = list()
        packing = dict()
        i = 0
        i_max = len(topology)
        while i < i_max:
            if topology[i] == '1':
                # slicing operation, super module
                sta.append([operation_variation.pop(0), 
                            topology_relations[i][0],
                            topology_relations[i][1],
                            topology_relations[i][2],
                            None, None, None,
                            None, None, None])
            else:
                # leaf cell, module
                module_name = module_permutation.pop(0)
                sta.append([module_name,
                            topology_relations[i][0],
                            topology_relations[i][1],
                            topology_relations[i][2],
                            0, 0, 0,
                            widths[module_name],
                            heights[module_name],
                            depths[module_name]])
            i += 1
        self.calc_dimensions(sta)
        i = 0
        i_max = len(sta)
        while i < i_max:
            if sta[i][STAE_TAG] in self.benchmark.blocks:
                packing[sta[i][STAE_TAG]] = [sta[i][STAE_X], 
                                             sta[i][STAE_Y], 
                                             sta[i][STAE_Z],
                                             sta[i][STAE_WIDTH],
                                             sta[i][STAE_HEIGHT],
                                             sta[i][STAE_DEPTH]]
            i += 1
        return packing


    def packing_to_representation(self, packing):
        """Creates a representation out of a packing.
        
        @param packing: a packing of a current representation
        @type packing: dictionary
        
        @note: is not possible with every data structure
        """
        print "Not implemented and/or available!"

    def get_operations(self):
        """Returns a dictionary of possible operations onto a abstract solution.
        The higher the globality_factor, the bigger is the solution change
        This factor is given by the authors but it also can be determined
        in test runs evaluation the cost influences caused by applying the
        according function.
        
        @return: Returns a dictionary of possible operations onto a abstract solution.
        structure: {'name' : [function, globality_factor]}
        @rtype: dictionary
        """
        return {'Exchange 2 modules' : [self.exchange_two_modules, 10],
                'Rotate a slicing plane' : [self.rotate_slicing_plane, 20],
                'Exchange 2 tree nodes' : [self.exchange_two_tree_nodes, 30],
                'Rotate single block in plane (keep z)': [self.rotate_module, 1]}

    def merge_representations(self, representation_a, representation_b):
        """Merges two representations into all possible children.
        Typically, used for genetic and/or evolutionary algorithms.
        Returns a list with all possible children.
        
        In case of Slicing Tree all possible combinations of the
        first, second and third sequence as well as the rotation_list
        are generated. Rotations are currently completely taken from
        representation a or b, respectively. Thus 2^4 different children
        are possible.
        
        @param representation_a: a representation list
        @type representation_a: list
        
        @param representation_b: a representation list
        @type representation_b: list
        
        @return: Returns a list with all possible representations created by merging the two given.
        @rtype: list
        """
        result = []
        for n in range(2**4):
            child = []
            for i in range(4):
                if n // 2**(3-i):
                    child.append(representation_a[i])
                else:
                    child.append(representation_b[i])
                n = n % 2**(3-i)
            result.append(child)
        return result
    
    def rotate_module(self, representation):
        """Rotates a module.
        @param representation: a representation list
        @type representation: list
        """
        rotations = list(representation[3])
        index = randint(0, len(rotations)-1)
        if rotations[index] == self.ORIGINAL:
            rotations[index] = self.ROTATED
        else:
            rotations[index] = self.ORIGINAL
        return representation[:3]+[rotations]

    def exchange_two_modules(self, representation):
        """Exchanges two modules.
        @param representation: a representation list
        @type representation: list
        """
        module_permutations = list(representation[1])
        pos1 = pos2 = randint(0, len(module_permutations)-1)
        while pos1 == pos2:
            pos2 = randint(0, len(module_permutations)-1)
        module_permutations[pos1], module_permutations[pos2] = module_permutations[pos2], module_permutations[pos1]
        return [representation[0]] + [module_permutations] + representation[2:]
        
    def rotate_slicing_plane(self, representation):
        """Rotates a slicing plane.
        @param representation: a representation list
        @type representation: list
        """
        operation_variation = list(representation[0])
        pos = randint(0, len(operation_variation)-1)
        shift = randint(1, 2)
        operations = ['X', 'Y', 'Z']
        old_index = operations.index(operation_variation[pos])
        new_index = (old_index + shift) % 3
        operation_variation[pos] = operations[new_index]
        return [operation_variation] + representation[1:]
        
    def exchange_two_tree_nodes(self, representation):
        """Exchanges two tree nodes.
        includes the special case where two modules are exchanged
        so maybe the exclusive module exchange function can be ommited
        @param representation: a representation list
        @type representation: list
        """
        slicings = representation[0]
        module_names = representation[1]
        topology = representation[2]
        first_pos = randint(1, len(topology)-1)
        counter = 2
        pos = 0
        ones = 1
        zeros = 0
        while pos < first_pos:
            pos = pos + 1
            if topology[pos] == '1':
                ones += 1
            else:
                zeros += 1
        if topology[pos] == '0':
            first_module_start = zeros-1
            first_module_end = zeros
            first_slicing_start = ones
            first_slicing_end = ones
        else:
            first_module_start = zeros
            first_slicing_start = ones-1
            while counter > 0:
                pos = pos + 1
                if topology[pos] == '1':
                    counter += 1
                    ones += 1
                else:
                    counter -=1
                    zeros += 1
            first_module_end = zeros
            first_slicing_end = ones
        first_end = pos
        second_choice = range(1, first_pos) + range(first_end+1, len(topology))
        overlap = True
        while overlap:
            overlap = False
            second_pos = second_choice[randrange(len(second_choice))]
            counter = 2
            pos = 0
            ones = 1
            zeros = 0
            while pos < second_pos:
                pos = pos + 1
                if topology[pos] == '1':
                    ones += 1
                else:
                    zeros += 1
            if topology[pos] == '0':
                second_module_start = zeros-1
                second_module_end = zeros
                second_slicing_start = ones
                second_slicing_end = ones
            else:
                second_module_start = zeros
                second_slicing_start = ones-1
                while counter > 0:
                    pos = pos + 1
                    if pos == first_pos:
                        second_choice.remove(second_pos)
                        overlap = True
                        break
                    if topology[pos] == '1':
                        counter += 1
                        ones += 1
                    else:
                        counter -=1
                        zeros += 1
                second_module_end = zeros
                second_slicing_end = ones
            second_end = pos
        if first_pos < second_pos:
            to = topology[:first_pos] + topology[second_pos:second_end] + \
            topology[first_end:second_pos] + topology[first_pos:first_end] + \
            topology[second_end:]
            mn = module_names[:first_module_start] + module_names[second_module_start:second_module_end] + \
            module_names[first_module_end:second_module_start] + module_names[first_module_start:first_module_end] + \
            module_names[second_module_end:]
            sl = slicings[:first_slicing_start] + slicings[second_slicing_start:second_slicing_end] + \
            slicings[first_slicing_end:second_slicing_start] + slicings[first_slicing_start:first_slicing_end] + \
            slicings[second_slicing_end:]
        else:
            to = topology[:second_pos] + topology[first_pos:first_end] + \
            topology[second_end:first_pos] + topology[second_pos:second_end] + \
            topology[first_end:]
            mn = module_names[:second_module_start] + module_names[first_module_start:first_module_end] + \
            module_names[second_module_end:first_module_start] + module_names[second_module_start:second_module_end] + \
            module_names[first_module_end:]
            sl = slicings[:second_slicing_start] + slicings[first_slicing_start:first_slicing_end] + \
            slicings[second_slicing_end:first_slicing_start] + slicings[second_slicing_start:second_slicing_end] + \
            slicings[first_slicing_end:]
        return [sl, mn, to] + representation[3:]

        
        