"""implements the GUI

This module is the main class of the GUI interface.
It contains the class DSC. If you create an object from this class the GUI will be started.
The GUI uses the PyQt4 Framework, so this must be installed on your computer.
This module has only been tested under windows OS.
It should work under different OS as well, but minor parts could need to be changed.

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""

#main uses modules
import os, sys, time
from PyQt4 import QtGui, QtCore
#from scipy.io.wavfile import read
path_cwd = os.getcwd()
path_cwd = os.path.split(path_cwd)[0]
path_cwd = os.path.split(path_cwd)[0]
sys.path.append(path_cwd)
#deletion for documentation
del path_cwd

#used to set the right paths --> for change edit module toolpaths
from dsc_suite.tools.toolpaths import FOLDER_FOR_DATA as FFDATA
from dsc_suite.tools.toolpaths import FOLDER_FOR_PLOTS as FFPLOTS
#information about content in the GUI
#these modules contain the import for the specific modules related to the content
from dsc_suite.gui.config import DATA_STRUCTURE_LIST, BENCHMARK_LIST, OPTIMIZATION_LIST, EVALUATION_PLOT_LIST
from dsc_suite.gui.config import HEADER_LIST, PARAMETER_LIST, DATATYPE_LIST, LIMITFUNCTION_LIST, READFUNCTION_LIST
from dsc_suite.gui.config import DIAGRAM_PARAMETER_LIST
from dsc_suite.ds.data_structure import COST_CRITERIA_LIST as COST_LIST
#classes used to generate or save data
from dsc_suite.gui.xmlpars import XMLParser
from dsc_suite.gui.generator import Generator
#only for old files
#from dsc_suite.analyses.characteristics import get_statistic_characteristics


class DSC(QtGui.QMainWindow):
    """Implements the GUI.
    This class is the main part of the GUI. It contains all functions to create the GUI interface.
    It is the only class in the project, which uses the PyQT Framework.
    The interfaces to other classes is based on Python standard data types as dictionary, list, etc.
    To start the GUI create an object of the DSC class and the GUI will start.
    
    @attention: The DSC starts an event bases loop. So the python programm which started the GUI will be terminated when the GUI is closed! An interactive console mode in the GUI is planned but not yet implemented. 
    """
    
    def __init__(self):
        """Creates a new instance of the GUI and starts the event routine.
        This function is the constructor of the DSC class.
        It calls the build up functions for the GUI and sets same basic configurations.
        It also launches the event loop.
        Attention: The constructor will not be leaved.
        """
        #application must be initialized before creating widgets
        self.__app = QtGui.QApplication(sys.argv)
        
        #create MainWindow, set title and make it central widget
        QtGui.QMainWindow.__init__(self)       
        self.setWindowTitle("DSC")
        self.__central = QtGui.QWidget()
        self.setCentralWidget(self.__central)
        self.setMinimumHeight(500)
        self.setMinimumWidth(1024)
        
        #create main layout for automatic fitting on window size
        self.__mainlayout = QtGui.QHBoxLayout(self.__central)
        
        #create a TabWidget
        self.__main_tab = QtGui.QTabWidget(self.__central)
        
        #create tabs
        list = ["Planer", "Evaluator"]
        #Managing Tab only for old files
        #"Managing"]
        self.__tabs = {}
        for string in list:
            tab = QtGui.QWidget(self.__main_tab)
            tab.setAutoFillBackground(True)
            tab.setMinimumWidth(400)
            self.__main_tab.addTab(tab, string)
            self.__tabs.update({string : tab})        
                
        #integrating MenuBar at top of window with tree structure
        self.__menubar = self.menuBar()
        self.__file = self.__menubar.addMenu("File")
        self.__action_new = self.__file.addMenu("New...")
        self.__action_new_data = self.__action_new.addAction("XML for data")
        self.__action_new_plot = self.__action_new.addAction("XML for plots")
        self.__action_open = self.__file.addAction("Open...")
        self.__action_exit = self.__file.addAction("Exit")
        
        #adding StatusBar for information messages 
        self.__statusbar = self.statusBar()
        
        #setting up default font  
        self.__font = QtGui.QFont()
        self.__font.setPointSize(9)
        #fill tabs with content
        self.__createDataView()
        
        self.__createPlaner()
        
        #was just for old files
        #self.__createManaging()
        
        #adding main widget to main layout
        self.__mainlayout.addWidget(self.__main_tab)
        
        #set main layout to MainWindow
        self.__central.setLayout(self.__mainlayout)
        
        #connecting slots and signals
        self.__connect()
        
        #setting standard XML File Paths
        self.__dataxml = os.path.join(FFDATA, "data.xml")
        self.__plotxml = os.path.join(FFPLOTS, "plots.xml")
        
        #creating new xml files if not existing
        if not os.path.exists(self.__dataxml):
            XMLParser(self.__dataxml, "datafile")
        if not os.path.exists(self.__plotxml):
            XMLParser(self.__plotxml, "plot")
        
        #create algorithm change criteria
        self.__current_algotihm = ""
            
        #load standard data file
        self.__refreshBrowser()
        #window ready
        self.__statusbar.showMessage("Ready")
        
        #show application and enter event routine
        self.showMaximized()
        sys.exit(self.__app.exec_())

    def __str__(self):
        """Returns a string with ID of the GUI object.
        @return: Returns a string with ID of the GUI object.
        @rtype: string
        """
        return "MainWindow with ID="+str(id(self))+" of " + str(self.__class__) 
    
    def keyPressEvent(self, event):
        """Overrides the original keyPressEvent().
        This function is used to perform actions when user presses specific keys.
        In the current version the the GUI not hotkeys are supported.
        
        @attention: standard windows key function need not to be implemented
        Example Alt+F4 closes the GUI without be implemented directly.
        
        @param event: a event generated by pressing a key
        @type event: QKeyEvent
        """
        if event.key() == QtCore.Qt.Key_F12:
            self.__statusbar.showMessage("Programmed by Tobias Heimpold")
                
    def timerEvent(self, event):
        """Overrides original timerEvent()
        This function is necessary for threading the calculation. It checks after a timer event whether the calculation
        is finished or running. It updates the process dialog.
        
        @param event: a event generated by a QTimer
        @type event: QTimerEvent
        """
        #check if calculation ready
        Generator.LockObject.acquire()
        flag = bool(self.__generator.isAlive())
        time_check = self.__generator.time_check
        self.__runDialog.setValue(self.__generator.current)
        Generator.LockObject.release()
        #if not ready leave...
        if flag:
            return
        else:
            #stop timer
            self.__timer.stop()
            if time_check:
                self.__showTimeEstimation()
            else:
                self.__finishCalc()
                
    def __createDataView(self):
        """Build the content of the Evaluator tab."""
        
        #create data file browser with group box
        self.__d_data_brow = QtGui.QGroupBox()
        self.__d_data_brow.setTitle("data file browser")
        self.__d_data_brow.setFont(self.__font)
        
        self.__d_data_browser = QtGui.QTreeWidget()
        self.__d_data_browser.setHeaderLabels(["data files", "runtime"])
        self.__d_data_browser.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.__d_data_browser.setMinimumWidth(700)
        
        #create information part for the single files
        self.__d_data_info = QtGui.QGroupBox()
        self.__d_data_info.setTitle("information")
        self.__d_data_info.setFont(self.__font)
        
        self.__d_data_params = QtGui.QTreeWidget()
        self.__d_data_params.setHeaderLabels(["parameter", "value"])
        self.__d_data_params.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        
        self.__d_data_char = QtGui.QTreeWidget()
        self.__d_data_char.setHeaderLabels(["characteristic", "value"])
        self.__d_data_char.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        
        #create evaluation part --> in this version a row of buttons
        self.__d_data_eval = QtGui.QGroupBox()
        self.__d_data_eval.setTitle("Evaluation options")
        self.__d_data_eval.setFont(self.__font)
                
        self.__eval_buttons = []
        
        #use data from config file to create the single buttons for evaluation
        buttons =  EVALUATION_PLOT_LIST.keys()
        buttons.sort()
        for string in buttons:
            button = QtGui.QPushButton()
            button.setFont(self.__font)
            button.setText(string)
            button.setToolTip(EVALUATION_PLOT_LIST[string][1])
            self.__eval_buttons.append(button)
        
        #linking evaluation function to button
        self.__eval_button_func = []
        
        for string in buttons:
            self.__eval_button_func.append(self.__gen_eval_func(string))
        #linking click event of button to function
        for i in range(0,len(self.__eval_buttons),1):
            self.connect(self.__eval_buttons[i], QtCore.SIGNAL("clicked()"), self.__eval_button_func[i])
        
        #adding buttons to a row             
        self.__evalbut_layout = QtGui.QHBoxLayout()
        self.__evalbut_layout.addStretch()
        for button in self.__eval_buttons:
            self.__evalbut_layout.addWidget(button)
        self.__evalbut_layout.addStretch()
        self.__d_data_eval.setLayout(self.__evalbut_layout)
        
        #putting the single parts together to the end layout
        self.__d_data_brow_layout = QtGui.QVBoxLayout()
        self.__d_data_brow.setLayout(self.__d_data_brow_layout)
        self.__d_data_brow_layout.addWidget(self.__d_data_browser)
        
        self.__d_data_info_layout = QtGui.QHBoxLayout()
        self.__d_data_info.setLayout(self.__d_data_info_layout)
        self.__d_data_info_layout.addWidget(self.__d_data_params)
        self.__d_data_info_layout.addWidget(self.__d_data_char)
        
        self.__d_right_layout = QtGui.QVBoxLayout()
        self.__d_right_layout.addWidget(self.__d_data_info)
        self.__d_right_layout.addWidget(self.__d_data_eval)
                
        self.__viewlayout = QtGui.QHBoxLayout(self.__tabs["Evaluator"])
        self.__viewlayout.addWidget(self.__d_data_brow)
        self.__viewlayout.addLayout(self.__d_right_layout)
    
    def __createPlaner(self):
        """Builds the content of the Planer tab."""
        #setting up default font  
        self.__font.setPointSize(10)
        
        #creating groupboxes with lists
        list = ["benchmarks", "datastructures", "algorithms"]
        dict = {"benchmarks" : BENCHMARK_LIST, "datastructures" : DATA_STRUCTURE_LIST , "algorithms" : OPTIMIZATION_LIST}
        infotext = {}
        #getting position of info text in config lsits
        for string in dict:
            for key in dict[string]:
                for item in dict[string][key]:
                    if type(item) == str:
                        infotext.update({string : dict[string][key].index(item)})
                        break
                break
        
        #ListWidgets with GroupBoxes and info text connection
        self.__p_groupboxes = {}
        self.__p_lists = {}
        
        for string in list:
            groupbox = QtGui.QGroupBox()
            groupbox.setTitle(string)
            groupbox.setFont(self.__font)
            groupbox.setMaximumHeight(250)
            layout = QtGui.QHBoxLayout()
            groupbox.setLayout(layout)
            listwid = QtGui.QListWidget()
            listwid.setMouseTracking(True)
            listwid.setFont(self.__font)
            layout.addWidget(listwid)
            for key in dict[string].keys():
                listwid.addItems([key])
            listwid.sortItems()
            #linking the information from config to the information part of the GUI
            self.connect(listwid, QtCore.SIGNAL("itemSelectionChanged ()"), self.__gen_info_func(listwid, dict[string], infotext[string]))
            self.__p_lists.update({string : listwid})
            self.__p_groupboxes.update({string : groupbox })
        
        #other GroupBoxes but with different layout
        list = ["settings", "information", "process priority", "planned trials"]
        layouts = {"settings" : QtGui.QHBoxLayout , "information" : QtGui.QHBoxLayout,  "process priority" : QtGui.QHBoxLayout, 
                   "planned trials": QtGui.QVBoxLayout}
        
        self.__p_groupboxes_layout = {}
        for string in list:
            groupbox = QtGui.QGroupBox()
            groupbox.setTitle(string)
            groupbox.setFont(self.__font)
            layout = layouts[string]()
            groupbox.setLayout(layout)
            self.__p_groupboxes.update({string : groupbox})
            self.__p_groupboxes_layout.update({string : layout})
        
        #planned trials table
        self.__p_planned_trials = QtGui.QTreeWidget()
        self.__p_planned_trials.setColumnCount(len(HEADER_LIST))
        self.__p_planned_trials.setHeaderLabels(HEADER_LIST)
        self.__p_planned_trials.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        #dictionary behind planned trials
        #used as interface to generator module !!!
        self.__generated_trials = {}
        
        self.__p_groupboxes_layout["planned trials"].addWidget(self.__p_planned_trials) 
        
        #defines a function to change the header of the planned trials table
        #opens a dialog with all parameters to choose from
        def changeHeader():
            dialog = QtGui.QDialog(self.__central)
            layout = QtGui.QVBoxLayout()
            dialog.setLayout(layout)
            checkboxes = []
            label = "global"
            groupbox = QtGui.QGroupBox(label)
            layout.addWidget(groupbox)
            sub_layout = QtGui.QHBoxLayout()
            groupbox.setLayout(sub_layout)
            list = HEADER_LIST
            current_header = []
            for column in range(self.__p_planned_trials.headerItem().columnCount()):
                current_header.append(str(self.__p_planned_trials.headerItem().text(column)))
                
            for string in list:
                box = QtGui.QCheckBox(string)
                sub_layout.addWidget(box)
                checkboxes.append(box)
                if string in current_header:
                    box.setChecked(True)
                
            for algorithm in OPTIMIZATION_LIST.keys():
                groupbox = QtGui.QGroupBox(algorithm)
                layout.addWidget(groupbox)
                sub_layout = QtGui.QHBoxLayout()
                groupbox.setLayout(sub_layout)
                for parameter in PARAMETER_LIST[algorithm].keys():
                    box = QtGui.QCheckBox(parameter)
                    sub_layout.addWidget(box)
                    checkboxes.append(box)
                    if parameter in current_header:
                        box.setChecked(True)
                sub_layout.addStretch()
            button = QtGui.QPushButton("Ok")
            button.setMaximumWidth(50)
            layout.addWidget(button)
            def change():
                dialog.close()
                new_header = []
                for box in checkboxes:
                    if box.isChecked():
                        new_header.append(box.text())
                self.__p_planned_trials.setHeaderLabels(new_header)
                self.__p_planned_trials.setColumnCount(len(new_header))
                for col in range(self.__p_planned_trials.columnCount()):
                    self.__p_planned_trials.resizeColumnToContents(col)
                self.__refreshPlaner()
                
                
            self.connect(button, QtCore.SIGNAL("clicked()"), change)
            dialog.show()

        #Buttons with tool tip and their functions
        list = ["Generate","Edit table view", "Start", "Remove", "Up", "Down"]
        dict = {"Generate": "Adds current selection to list",
                "Start": "Starts the calculation",
                "Edit table view" : "Changes the header of the table",
                "Remove" : "Removes selected item from list",
                "Up" : "Moves item up in list",
                "Down" : "Moves item down in list"}
        func_dict = {"Generate" : self.__generate, 
                     "Start" : self.__startCalc,
                     "Edit table view" : changeHeader,
                     "Remove" : self.__gen_index_change_func("Remove"),
                     "Up" : self.__gen_index_change_func("Up"),
                     "Down" :self.__gen_index_change_func("Down")}
        
        #create buttons
        self.__p_pushbuttons = {}
        for string in list:
            button = QtGui.QPushButton()
            button.setText(string)
            button.setFont(self.__font)
            button.setToolTip(dict[string])
            button.setEnabled(True)
            self.__p_pushbuttons.update({string: button})
            self.connect(button, QtCore.SIGNAL("clicked()"), func_dict[string])
        
        self.__p_pushbuttons["Start"].setEnabled(False)
            
        #InfoText label --> views information from config file
        self.__plabel_infotext = QtGui.QLabel()
        self.__plabel_infotext.setFont(self.__font)
        self.__plabel_infotext.setMaximumHeight(150)       
            
        #Radio Buttons for priority choise in GroupBox
        #just works with windows OS at the moment --> see generator module for information
        list = ["Low", "Less normal", "Normal"]
        dict = {"Low": "using idle processor time", "Less normal" : "background" , "Normal" : ""}
        self.__p_radiobuttons = {}
        self.__radiobutton_group = QtGui.QButtonGroup()
        for string in list:
            button = QtGui.QRadioButton()
            button.setText(string)
            button.setToolTip(dict[string])
            button.setFont(self.__font)
            self.__p_radiobuttons.update({string : button})
            self.__radiobutton_group.addButton(button, list.index(string))
            self.__p_groupboxes_layout["process priority"].addWidget(button)
            
        self.__p_radiobuttons["Normal"].setChecked(True)
               
        #vertical layout (mainlayout for planer tab)
        self.__hmainbox = QtGui.QHBoxLayout(self.__tabs["Planer"])
        self.__tabs["Planer"].setLayout(self.__hmainbox)
        
        #horizontal sub-layouts (left and right) with line
        self.__vmainbox_r = QtGui.QVBoxLayout()
        self.__vmainbox_l = QtGui.QVBoxLayout()
        self.__vline = QtGui.QFrame()
        self.__vline.setFrameShape(QtGui.QFrame.VLine)
        self.__vline.setFrameShadow(QtGui.QFrame.Sunken)
        
        #horizontal sub-widgets
        self.__vmainwid_l = QtGui.QWidget()
        self.__vmainwid_l.setLayout(self.__vmainbox_l)
        self.__vmainwid_l.setMaximumWidth(450)
        self.__vmainwid_r = QtGui.QWidget()
        self.__vmainwid_r.setLayout(self.__vmainbox_r)
        
        
        #adding sub-layouts to main layout
        self.__hmainbox.addWidget(self.__vmainwid_l)
        self.__hmainbox.addWidget(self.__vline)
        self.__hmainbox.addWidget(self.__vmainwid_r)
               
        #grid-layout for left side    
        self.__grid = QtGui.QGridLayout()
        self.__grid.setSizeConstraint(QtGui.QLayout.SetMinimumSize)
        self.__vmainbox_l.addLayout(self.__grid)    
                                        
        #Buttons
        self.__p_button_layout = QtGui.QHBoxLayout()
        list = ["Edit table view","","Remove", "Up", "Down", "Start"]
        for string in list:
            if string == "":
                self.__p_button_layout.addStretch()
            else:
                self.__p_button_layout.addWidget(self.__p_pushbuttons[string])
                         
        #adding widgets and filled layouts
        #adding in-grid layouts to grid-layout
        self.__grid.addWidget(self.__p_groupboxes["benchmarks"],0,0)
        self.__grid.addWidget(self.__p_groupboxes["datastructures"],0,1)
        self.__grid.addWidget(self.__p_groupboxes["algorithms"],0,2)    
        
        #self.__p_groupboxes_layout["parameters"].addWidget(self.__p_pushbuttons["Generate"])         
        self.__p_groupboxes_layout["information"].addWidget(self.__plabel_infotext)
        
               
        self.__vmainbox_r.addWidget(self.__p_groupboxes["planned trials"])
        self.__vmainbox_r.addLayout(self.__p_button_layout)
                
        self.__vmainbox_l.addWidget(self.__p_groupboxes["settings"])
        self.__vmainbox_l.addWidget(self.__p_groupboxes["information"])
        self.__vmainbox_l.addWidget(self.__p_groupboxes["process priority"])
        
        #separated function to better the reading of code --> changes of parameter must only be considered there      
        self.__createSettingSection()
        
    def __createSettingSection(self):
        """Builds the content of the parameter section of the Planer Tab.
            - changes of parameters must be only considered here
            - creates different layouts for the specific parameters of the algorithms
            - it links the parameters with their name with an dictionary
            - to get the parameter inserted by the user --> use that dictionary
        """
        
        #create scroll area for settings sector
        scroll_area = QtGui.QScrollArea()
        scroll_area.setFrameStyle(QtGui.QFrame.NoFrame)
        widget = QtGui.QWidget()
        widget.setMinimumSize(2, 2)
        scroll_area.setWidget(widget)
        area_layout = QtGui.QGridLayout()
        area_layout.setSizeConstraint(QtGui.QLayout.SetMinAndMaxSize)
        widget.setLayout(area_layout)
        self.__p_groupboxes_layout["settings"].addWidget(scroll_area)
        
        #differs in general and algorithm specific paramters      
        list = ["general" , "individual"]
        dict = {"general" : QtGui.QVBoxLayout, "individual" : QtGui.QStackedLayout}
        
        self.__p_setting_boxes_layout = {}
        
        for string in list:
            box = QtGui.QGroupBox()
            box.setTitle(string)
            layout = dict[string]()
            box.setLayout(layout)
            self.__p_setting_boxes_layout.update({string : layout})
            area_layout.addWidget(box,0,list.index(string))
        area_layout.addWidget(self.__p_pushbuttons["Generate"],1,1, alignment = QtCore.Qt.AlignBottom)
        
        self.__p_general_settings = {}
        
        #list of general parameters
        list = ["trial name", "z-D" , "cost criteria" , "weight factors", "norm values", "samples for norm values"]
        tool = {"trial name" : "the name for the trial",
                "z-D" : "third dimension of block, auto generated or specific value",
                "cost criteria": "choose at least one cost criteria to experiment",
                "weight factors": "set the influence of cost criteria above to cost function",
                "norm values" : "used to normalize the values of the cost criteria, loaded from database or estimated from samples",
                "samples for norm values" : "samples used to calculate the mean values of the cost criteria"}
        
        self.__p_general_set_boxes_layouts = {}
        
        for string in list:
            box = QtGui.QGroupBox()
            layout = QtGui.QHBoxLayout()
            box.setLayout(layout)
            box.setTitle(string)
            box.setToolTip(tool[string])
            self.__p_setting_boxes_layout["general"].addWidget(box)
            self.__p_general_set_boxes_layouts.update({string : layout})
            
        #trial name
        tnline =QtGui.QLineEdit()
        self.__p_general_set_boxes_layouts["trial name"].addWidget(tnline)
        self.__p_general_set_boxes_layouts["trial name"].addWidget(tnline)
        self.__p_general_settings.update({"trial name":tnline})
        
        #third dimension component for 2D benchmarks
        zcheck = QtGui.QCheckBox("auto")
        zline = QtGui.QLineEdit()
        self.__p_general_set_boxes_layouts["z-D"].addWidget(zcheck)
        self.__p_general_set_boxes_layouts["z-D"].addWidget(zline)
        self.__p_general_settings.update({"z-D":zline})
        
        def auto3D():
            if zcheck.isChecked():
                zline.setEnabled(False)
            else:
                zline.setEnabled(True)
        self.connect(zcheck, QtCore.SIGNAL("stateChanged (int)"),auto3D)
        zcheck.setChecked(True)
        
        #cost criteria which are implemented in data structure module
        #also added an weight factor for optimization functions
        #factors should be checked if their sum=1 !
        for criteria in COST_LIST.keys():
            check_box = QtGui.QCheckBox()
            check_box.setText(criteria)
            spin_box = QtGui.QDoubleSpinBox()
            spin_box.setRange(0.0,1.0)
            spin_box.setValue(0.5)
            spin_box.setDecimals(2)
            spin_box.setSingleStep(0.05)
            spin_box.setEnabled(False)
            self.__p_general_set_boxes_layouts["cost criteria"].addWidget(check_box)
            self.__p_general_set_boxes_layouts["weight factors"].addWidget(spin_box)
            self.__p_general_settings.update({criteria : check_box})
            self.__p_general_settings.update({"weight_"+criteria : spin_box})           
            def defineEnableFunction(criteria):
                def enableBox():
                    if self.__p_general_settings[criteria].isChecked():
                        self.__p_general_settings["weight_"+criteria].setEnabled(True)
                    else:
                        self.__p_general_settings["weight_"+criteria].setEnabled(False)
                return enableBox     
            self.connect(self.__p_general_settings[criteria], QtCore.SIGNAL("stateChanged (int)"), defineEnableFunction(criteria))
        
        #mean value of the cost criteria for normalization of cost function
        #mean values can be uses from Monte Carlo results or can be estimated with a short run of Monte Carlo   
        load_button = QtGui.QRadioButton("database")
        new_button = QtGui.QRadioButton("estimate")
        radio_but_gro = QtGui.QButtonGroup()
        radio_but_gro.addButton(load_button,0)
        radio_but_gro.addButton(new_button,1)
        spin_box = QtGui.QSpinBox()
        spin_box.setRange(1,10000)
        spin_box.setValue(5000)
        spin_box.setSingleStep(100)
        self.__p_general_set_boxes_layouts["samples for norm values"].addWidget(spin_box)
        self.__p_general_settings.update({"mean_samples" : spin_box,
                                          "estimate_button" : new_button})
        #mean samples for estimation should only be enables when button is really "estimate"
        def buttonchange():
            a = radio_but_gro.checkedId()
            if a == 0:
                spin_box.setEnabled(False)
            if a == 1:
                spin_box.setEnabled(True)
        
        self.connect(load_button, QtCore.SIGNAL("clicked ()"), buttonchange)
        self.connect(new_button, QtCore.SIGNAL("clicked ()"), buttonchange)
        load_button.click()
        
        self.__p_general_set_boxes_layouts["norm values"].addWidget(load_button)
        self.__p_general_set_boxes_layouts["norm values"].addWidget(new_button)
        
        #creating algorithm specific parameter --> uses actual selected algorithm
        #config file contains the different parameters, their data type, the class of uses PyQt interface and the functions which changes the parameter configurations
        # --> see config file to understand the next parts !!!
        self.__p_individual_settings= {}
        for key in PARAMETER_LIST.keys():
            self.__p_individual_settings.update({key: {}})
        self.__p_individual_widgets = {}
        no_widget = self.__p_setting_boxes_layout["individual"].addWidget(QtGui.QWidget())
        for algorithm in PARAMETER_LIST.keys():
            widget = QtGui.QWidget()
            top_layout = QtGui.QVBoxLayout()
            widget.setLayout(top_layout)
            a = self.__p_setting_boxes_layout["individual"].addWidget(widget)
            self.__p_individual_widgets.update({algorithm : a})     
            
            for parameter in PARAMETER_LIST[algorithm].keys():
                datatype = PARAMETER_LIST[algorithm][parameter][0]
                tooltip = PARAMETER_LIST[algorithm][parameter][1]
                limits = PARAMETER_LIST[algorithm][parameter][2]
                box = QtGui.QGroupBox()
                box.setTitle(parameter.replace("_"," "))
                box.setToolTip(tooltip)
                top_layout.addWidget(box)
                layout = QtGui.QVBoxLayout()
                box.setLayout(layout)
                widget = eval(DATATYPE_LIST[datatype])
                layout.addWidget(widget)
                function_list = LIMITFUNCTION_LIST[datatype]
                
                self.__p_individual_settings[algorithm].update({parameter : widget})
                
                for index in range(len(function_list)):
                    func = "widget."+function_list[index]
                    eval(func)(limits[index]) 
                      
            top_layout.addStretch()
        self.__p_setting_boxes_layout["individual"].setCurrentIndex(no_widget)
        
        
    def __changeParamLayout(self):
        """Switches between the different algorithm parameter layouts"""        
        a = str(self.__p_lists["algorithms"].currentItem().text())
        self.__p_setting_boxes_layout["individual"].setCurrentIndex(self.__p_individual_widgets[a])
        
    '''only for old files
    def __createManaging(self):
        """__ccreatesManaging
        - this section was implemented to import old data files into an XML file
        - the work has been halted after the important files were integrated
        
        The documentation of this part is still missing.
        The basis structure is:
        - get file name
        - try to find information in the file name --> present them to user for check
        - calculate the statistic data from the file content for XML file
        - create new entry in XML file
        
        Attention:
        Because of a different method to save information in the data file name, this function must be adjusted by hand and is very buggy !
        It also works only for one file only. For new files use the Planer Part of the GUI !
        """ 
        self.__m_mainlayout = QtGui.QGridLayout()
        self.__tabs["Managing"].setLayout(self.__m_mainlayout)
        
        list = ["Import old datafiles", "Edit parameters", "benchmark", "datastructure", "algorithm", "samples", "cost criteria", "runtime",
                "z-D", "Characteristics", "Update datafile", "How to use"]
        layouts = {"Import old datafiles" : QtGui.QVBoxLayout, "Edit parameters" : QtGui.QVBoxLayout, "benchmark" : QtGui.QVBoxLayout,
                   "datastructure": QtGui.QVBoxLayout, "algorithm": QtGui.QVBoxLayout, "samples": QtGui.QVBoxLayout, "cost criteria" : QtGui.QVBoxLayout,
                   "runtime": QtGui.QVBoxLayout, "z-D": QtGui.QVBoxLayout, "Characteristics" : QtGui.QVBoxLayout, "Update datafile" : QtGui.QHBoxLayout,
                   "How to use": QtGui.QVBoxLayout}
        
        self.__m_groupboxes = {}
        self.__m_groupboxes_layouts = {}
        
        for string in list:
            groupbox = QtGui.QGroupBox()
            groupbox.setTitle(string)
            groupbox.setFont(self.__font)
            layout = layouts[string]()
            groupbox.setLayout(layout)
            self.__m_groupboxes.update({string:groupbox})
            self.__m_groupboxes_layouts.update({string: layout})
        
        list = ["filename", "benchmark", "datastructure", "algorithm", "samples","cost criteria", "runtime", "z-D"]
        
        self.__m_inputElements = {}
        
        for string in list:
            widget = QtGui.QLineEdit()
            widget.setMaximumWidth(200)
            widget.setFont(self.__font)
            self.__m_inputElements.update({string : widget})
        
        list = ["benchmark", "datastructure", "algorithm", "samples","cost criteria", "runtime", "z-D"]
        
        for string in list:
            self.__m_groupboxes_layouts[string].addWidget(self.__m_inputElements[string])
            self.__m_groupboxes_layouts["Edit parameters"].addWidget(self.__m_groupboxes[string]) 
        
        list = ["Select file", "Characteristics", "Write to XML"]
        dict = {"Select file" : "Opens a dialog to select datafile", "Characteristics" : "Calculates the characteristics", "Write to XML" : "Writes the information into the selected XML file"}
        
        self.__m_pushbuttons = {}
        for string in list:
            widget = QtGui.QPushButton()
            widget.setFont(self.__font)
            widget.setText(string)
            widget.setMaximumWidth(150)
            widget.setToolTip(dict[string])
            self.__m_pushbuttons.update({string : widget})
        
        self.__m_charact_groupboxes = {}
        self.__m_charact_labels = {}
        
        def getcharacts():

            path = os.path.split(str(self.__m_inputElements["filename"].text()))[1]
            result = get_statistic_characteristics(path)
            for key in result.keys():
                if key in self.__m_charact_groupboxes.keys():
                    self.__m_charact_labels[key].setText(str(result[key]))
                    continue
                self.__m_charact_groupboxes.update({key: QtGui.QGroupBox()})
                self.__m_charact_groupboxes[key].setFont(self.__font)
                self.__m_charact_groupboxes[key].setTitle(key)
                layout = QtGui.QHBoxLayout()
                self.__m_charact_groupboxes[key].setLayout(layout)
                self.__m_charact_labels.update({key : QtGui.QLabel()})
                self.__m_charact_labels[key].setFont(self.__font)
                self.__m_charact_labels[key].setText(str(result[key]))
                layout.addWidget(self.__m_charact_labels[key])
                self.__m_groupboxes_layouts["Characteristics"].addWidget(self.__m_charact_groupboxes[key])
        
        self.__m_guide_label = QtGui.QLabel()
        string = ("1. Use 'Select file' button to choose a datafile.\n"
                  "2. The program tries to auto-complete the parameter lines on the right site.\n"
                  "3. Check the found parameters.\n"
                  "4. (Optional) Correct or complete missing parameters.\n"
                  "5. (Optional) You can calculate the characteristics by pressing the 'Characteristics button'.\n"
                  "6. Click the 'Write to XML' button to link the file with the current XML database.\n"
                  "\n"
                  "Addition:\n"
                  "If the 'z-D' field is left blank, the programm will set the parameter to 'auto'.\n"
                  "The 'runtime' field need not to be filled.")
                 
        self.__m_guide_label.setText(string)
        
        
        self.connect(self.__m_pushbuttons["Characteristics"], QtCore.SIGNAL("clicked()"), getcharacts)
        
        self.__m_groupboxes_layouts["Import old datafiles"].addWidget(self.__m_inputElements["filename"])
        self.__m_groupboxes_layouts["Import old datafiles"].addWidget(self.__m_pushbuttons["Select file"])
        self.__m_groupboxes_layouts["Import old datafiles"].addWidget(self.__m_groupboxes["How to use"])
        self.__m_groupboxes_layouts["Import old datafiles"].addStretch()
        
        self.__m_groupboxes_layouts["Update datafile"].addWidget(self.__m_pushbuttons["Characteristics"])
        self.__m_groupboxes_layouts["Update datafile"].addWidget(self.__m_pushbuttons["Write to XML"])
        
        self.__m_groupboxes_layouts["How to use"].addWidget(self.__m_guide_label)
        
        self.__m_mainlayout.addWidget(self.__m_groupboxes["Import old datafiles"],0,0)
        self.__m_mainlayout.addWidget(self.__m_groupboxes["Edit parameters"], 0,1)
        self.__m_mainlayout.addWidget(self.__m_groupboxes["Characteristics"],1,0)
        self.__m_mainlayout.addWidget(self.__m_groupboxes["Update datafile"],1,1)
        '''      
    def __connect(self):
        """Connects the widgets with signals and slots.
        @attention: Not every signals and slots can be connected in this global way. But those who can should be done here ! If not the code gets very difficult to read.
        """
        #connecting Slots and Signals with function - menu
        self.connect(self.__action_exit, QtCore.SIGNAL("triggered()"), QtGui.qApp, QtCore.SLOT("quit()"))
        self.connect(self.__action_open, QtCore.SIGNAL("triggered()"), self.__openFile)
        self.connect(self.__action_new_data, QtCore.SIGNAL("triggered()"), self.__newXMLDataFile)
        self.connect(self.__action_new_plot, QtCore.SIGNAL("triggered()"), self.__newXMLPlotFile)
        
        #connecting Slots and Signals with function - planer area
        self.connect(self.__p_lists["algorithms"], QtCore.SIGNAL("itemClicked (QListWidgetItem *)"), self.__changeParamLayout)
        
        #connecting Slots and Signals with function - data area
        self.connect(self.__d_data_browser, QtCore.SIGNAL("itemSelectionChanged ()"), self.__datainfo)
        self.connect(self.__d_data_browser, QtCore.SIGNAL("itemExpanded (QTreeWidgetItem *)"), self.__dataview_fit)
        self.connect(self.__d_data_browser, QtCore.SIGNAL("itemCollapsed (QTreeWidgetItem *)"), self.__dataview_fit)
        
        #connecting Slots and Signals with function - manager area
        #only for old files
        #self.connect(self.__m_pushbuttons["Select file"], QtCore.SIGNAL("clicked ()"), self.__openFile)
        #self.connect(self.__m_pushbuttons["Write to XML"], QtCore.SIGNAL("clicked ()"), self.__updateXML)
       
    def __newFile(self, string):
        """Used to create new XML files.
        @param string: the string is used to differ between data xml and plot xml file
        @type string: string
        """
        filedialog = QtGui.QFileDialog(self)
        path = ""
        if string == "datafile":
            path = str(filedialog.getSaveFileName(self, "Save File", FFDATA, "XML (*.xml)"))
            self.__dataxml = path
        if string == "plot":
            path = str(filedialog.getSaveFileName(self, "Save File", FFPLOTS, "XML (*.xml)"))
            self.__plotxml = path
        if path == "":
            return
        XMLParser(path, string)
        self.__refreshBrowser()
    
    def __newXMLDataFile(self):
        """Used to call newFile with data file creation."""
        self.__newFile("datafile")
    
    def __newXMLPlotFile(self):
        """Used to call newFile with plot file creation."""
        self.__newFile("plot")
                
    def __openFile(self):
        """Opens files.        
        Old version: There was an option to save the generated trials to an text file.
        This feature is not yet implemented after the complete planer tool was re-designed
        """
        #using QFileDialog
        filedialog = QtGui.QFileDialog(self)
        #just allow data files for current tab
        tabtext = self.__main_tab.tabText(self.__main_tab.currentIndex())
        if tabtext == "Evaluator":
            self.__openXMLFile(str(filedialog.getOpenFileName(self, "Open File", FFDATA, "XML (*.xml)" )))
        if tabtext == "Planer":
            path = (str(filedialog.getOpenFileName(self, "Open File", FFDATA, "XML (*xml);;SCHED (*.sched)" )))
            if os.path.splitext(path)[1] == ".xml":
                self.__openXMLFile(path)
            if os.path.splitext(path)[1] == ".SCHED":
                self.__openSCHEDfile(path)
        if tabtext == "Managing":
            path = (str(filedialog.getOpenFileName(self, "Open File", FFDATA)))
            if path == "":
                return
            self.__import(path)
             
    def __openXMLFile(self, path):
        """Used to open XML files.
        @param path: the path to the new XML file, real path
        @type path: string
        
        @attention: Large file will take same time to load.
        """
        #if canceled abort    
        if path == "":
            return
        parser = XMLParser(path, "")
        if parser.getRootName() == "datafiles":
            self.__dataxml = path
            self.__refreshBrowser()
        elif parser.getRootName() == "plots":
            self.__plotxml = path
        else:
            self.__popUp("XML file could not be loaded.\nThe rootname differs from the defaults.")      
        self.__statusbar.showMessage("XML File loaded")
    
    def __infoText(self, str):
        """Shows the string in the information area in planer tab.
        @param str: information as string, must be pretty print formatted
        @type str: string"""
        self.__plabel_infotext.setText(str)
        
    def __generate(self):
        """Generates a table entry in planer tab with the selected parameters and creates a dictionary which is used as interface for the generator module."""
        #no dictionary --> create new
        try:
            self.__generated_trials.keys()
        except:
            self.__generated_trials =  {}
            
        #items in list selected
        new_index = len(self.__generated_trials.keys())
        
        self.__generated_trials.update({new_index : {"init_params" : {},
                                                    "individual_params" : {}}})
        
        self.__current_index_dict = self.__generated_trials[new_index]
        
        #check selection --> something missing --> error
        try:
            data = str(self.__p_lists["datastructures"].currentItem().text())
        except AttributeError:
            del(self.__generated_trials[new_index])
            self.__popUp("Missing data structure")
            return
        try:
            bench = str(self.__p_lists["benchmarks"].currentItem().text())
        except AttributeError:
            del(self.__generated_trials[new_index])
            self.__popUp("Missing benchmark")
            return
        try:
            algorithm = str(self.__p_lists["algorithms"].currentItem().text())
        except AttributeError:
            del(self.__generated_trials[new_index])
            self.__popUp("Missing algorithm")
            return
        
        #add information to dictionary                
        self.__current_index_dict["init_params"].update({"benchmark" : bench,
                                                         "datastructure" : data,
                                                         "algorithm" : algorithm,
                                                         "cost_criteria" : {"SUM" : {}}})
        #add trial name
        try:
            name = str(self.__p_general_settings["trial name"].text())
            if name=="":
                self.__popUp("No name given. Data files will be named with time stamp.")
                name= time.strftime("%Y-%m-%d")
            if "\\"in name or "/"in name or ":"in name or "*"in name or "?"in name or "\""in name or "<"in name or ">"in name or "|"in name:
                self.__popUp("Name must not have litarals: \\ / : * ? \" < > |")
                return
        except:
            self.__popUp("Name not readable. Data files will be named with time stamp.")
            name=time.strftime("%Y-%m-%d")
        self.__current_index_dict["init_params"].update({"trial_name" : name})
        
        #add trial date
        self.__current_index_dict["init_params"].update({"trial_date" : time.strftime("%Y-%m-%d")})
        
        #add 3D info for 2D benchmarks
        if self.__p_general_settings["z-D"].isEnabled():
            try:
                value = float(self.__p_general_settings["z-D"].text())
            except:
                self.__popUp("Can not evaluate z-D parameter")
            self.__current_index_dict["init_params"].update({"z-Dimension" : value})
        else:
            self.__current_index_dict["init_params"].update({"z-Dimension" : "auto"})
        
        #getting information about cost criteria and their weight factors  
        sum = 0.0
        for criteria in COST_LIST.keys():
            if self.__p_general_settings[criteria].isChecked():
                self.__current_index_dict["init_params"]["cost_criteria"].update({criteria : {}})
                factor = round(float(self.__p_general_settings["weight_"+criteria].value()),2)
                self.__current_index_dict["init_params"]["cost_criteria"]["SUM"].update({"weight_factor_"+criteria: factor})
                sum += factor
        sum = round(sum, 2)
        
        #checking sum of weight factors, should be 1
        if sum != 1.0:
            self.__popUp("sum of weight factors not equal 1.0")
            del(self.__generated_trials[new_index])                
            return
        #checking for database of Monte Carlo results
        if not os.path.exists(FFDATA+"\mean_values.xml"):
            self.__p_general_settings["estimate_button"].click()
            self.__p_general_settings["mean_samples"].setValue(5000)
            
        if self.__p_general_settings["mean_samples"].isEnabled():
            value = int(self.__p_general_settings["mean_samples"].value())
            self.__current_index_dict["init_params"].update({"mean_samples" : value})
        else:
            keys = set(self.__current_index_dict["init_params"]["cost_criteria"].keys())
            cost_criteria = list(keys & set(COST_LIST.keys()))
            self.__current_index_dict["init_params"].update({"mean_samples" : None})
            parser = XMLParser(FFDATA+"\mean_values.xml", "datafile")
            data_structures = ['3D Slicing Tree', 'Sequence Triple', 'Sequence Quintuple', 'O-Sequence', 'T-Tree']
            norm_values = dict.fromkeys(cost_criteria, None)
            for ds in data_structures:
                maxima = parser.getCharacteristic(ds, "Monte Carlo", bench, cost_criteria, ["maximum"])
                for crit in cost_criteria:
                    if norm_values[crit] != None:
                        try:     
                            if float(norm_values[crit]) < float(maxima[crit]["maximum"]):
                                norm_values[crit] = maxima[crit]["maximum"]
                        except:
                            print "Attention! Norm value search error detected"
                            self.__popUp("Attention! Norm value search error detected\n"
                                         "Check console output for more information.")
                            break
                    else:
                        norm_values[crit] = maxima[crit]["maximum"]
            if None in norm_values.values():
                self.__popUp("Can not find combination of data structure and benchmark in database")
                del(self.__generated_trials[new_index])                
                return
            for criteria in cost_criteria:
                value = float(norm_values[criteria])
                self.__current_index_dict["init_params"]["cost_criteria"]["SUM"].update({"norm_value_"+criteria : value})
        #getting specific parameters for algorithm
        #no error correction --> config file should provide a basis configuration of these parameters
        #read function of the parameter also defined in config file             
        for parameter in PARAMETER_LIST[algorithm].keys():
            datatype = PARAMETER_LIST[algorithm][parameter][0]
            function = READFUNCTION_LIST[datatype]
            widget = self.__p_individual_settings[algorithm][parameter]
            string = datatype+"(widget."+function+"())"
            value = eval(string)
            self.__current_index_dict["individual_params"].update({parameter : value})
        
        #refresh the table of created trials   
        self.__refreshPlaner()
        #setting the start button to enabled
        self.__p_pushbuttons["Start"].setEnabled(True)
        
        self.__statusbar.showMessage("Trial added")
        
    def __refreshPlaner(self):
        """Used to display the generated trials in the table of the planer tab."""
        
        #clearing actuall table
        self.__p_planned_trials.clear()
        #setting header
        current_header = self.__p_planned_trials.headerItem()
        #generate new entries for table
        #uses header as basis and gets information from dict and transforms them into pretty print
        for index in self.__generated_trials.keys():
            new_item = QtGui.QTreeWidgetItem()
            self.__p_planned_trials.addTopLevelItem(new_item)
            init_params = self.__generated_trials[index]["init_params"]
            individual_params = self.__generated_trials[index]["individual_params"]
            cost_criteria = self.__generated_trials[index]["init_params"]["cost_criteria"]
            sum_criteria = self.__generated_trials[index]["init_params"]["cost_criteria"]["SUM"]
               
            for col in range(current_header.columnCount()):
                text = str(current_header.text(col))
                if text == "#":
                    new_item.setText(col, str(index))
                elif text.replace(" ","_") in init_params:
                    new_item.setText(col, str(init_params[text.replace(" ","_")]))
                elif text in individual_params:
                    new_item.setText(col, str(individual_params[text]))
                elif text.replace(" ","_") in sum_criteria:
                    new_item.setText(col, str(sum_criteria[text.replace(" ","_")]))
                elif text in cost_criteria:
                    new_item.setText(col, "+")
                new_item.setTextAlignment(col, QtCore.Qt.AlignCenter)
        #resize the columns to fit to data          
        for col in range(self.__p_planned_trials.columnCount()):
            self.__p_planned_trials.resizeColumnToContents(col)
    
    def __plannedChangeIndex(self, string):
        """Used to change the order in the planned trial table.
            - this function is used to change the order in the table of the planer tab
            - it edits the dictionary not the table !
        @param string: must be one of the following:
            - Remove :  removes selected entry
            - Up: moves the entry up --> lower index
            - Down: moves the entry down --> higher index
        @type string: string
        """
        #check if something is selected
        try:
            item = self.__p_planned_trials.selectedItems()[0]
            index = int(str(item.text(0)))
            last_index = len(self.__generated_trials.keys())-1
        except:
            self.__popUp("No item selected")
            return
        #remove item
        if string == "Remove":
            for i in range(index,len(self.__generated_trials.keys())-1, 1):
                self.__generated_trials[i] = self.__generated_trials[i+1]
            del(self.__generated_trials[last_index])
        #switch positions --> decrement index
        elif string == "Up" and index > 0:
            temp = self.__generated_trials[index-1]
            self.__generated_trials[index-1] = self.__generated_trials[index]
            self.__generated_trials[index] = temp
            index -= 1
        #switch positions --> increment index
        elif string == "Down" and index < last_index:
            temp = self.__generated_trials[index+1]
            self.__generated_trials[index+1] = self.__generated_trials[index]
            self.__generated_trials[index] = temp
            index += 1
        self.__refreshPlaner()
        if len(self.__generated_trials.keys()) > 0:
            item = self.__p_planned_trials.topLevelItem(index)
            self.__p_planned_trials.setItemSelected(item, True)
        #disable Start button if table empty
        else:
            self.__p_pushbuttons["Start"].setEnabled(False)
            
    def __refreshBrowser(self):
        """Reads the information from a XML file and updates the DataBrowser.
            - this function is used to read the data from the given XML file as parser
            - it reads the data and creates entries to a TreeWidget
            - it also generates dictionaries with the statistic information of every file
            - this dictionary will be displayed when the file is selected in the browser
        
        @attention:: Larger datafiles need time to load --> very big files need several minutes. The OS can determine a hang up for the GUI. Do not kill it.
        """
        #clear old data
        self.__d_data_browser.clear()
        self.__d_data_params.clear()
        self.__d_data_char.clear()
        parser = XMLParser(self.__dataxml, "datafile")
        
        #dictionaries for tree
        #top level of tree are data structures
        #second level are algorithms
        #third level are benchmarks
        #last level are the single files
        self.__d_datastructure_items = {}
        self.__d_algorithm_items = {}
        self.__d_benchmark_items = {}
        self.__d_trial_name_items = {}
        self.__d_file_items = {}
        self.__dtreeinfotext = {}
        self.__dtreeparamtext = {}
        self.__d_file_information = {}
        
        #getting entries from XML file
        for i in range(parser.countTopLevelTags()):
            dict = parser.nextTopLevelTag(i)
            #get main information
            try:
                datastructure = dict["datastructure"]
                algorithm = dict["algorithm"]
                benchmark = dict["benchmark"]
                trial_name = dict["trial_name"]
                cost_criteria = dict["cost_criteria"]
                trial_date = dict["trial_date"]
                runtime = dict["runtime"]
                z_dimension = dict["z-Dimension"]
            except:
                self.__popUp("Error! XML file could not be read!")
                return
            #create entries for dictionaries
            #avoids doubled entries
            if datastructure not in self.__d_datastructure_items.keys():
                toplevelitem = QtGui.QTreeWidgetItem()
                toplevelitem.setText(0, datastructure)
                self.__d_data_browser.addTopLevelItem(toplevelitem)
                self.__d_datastructure_items.update({datastructure : toplevelitem})
            if datastructure+algorithm not in self.__d_algorithm_items.keys():
                item = QtGui.QTreeWidgetItem()
                item.setText(0, algorithm)
                self.__d_datastructure_items[datastructure].addChild(item)
                self.__d_algorithm_items.update({datastructure+algorithm : item})
            if datastructure+algorithm+benchmark not in self.__d_benchmark_items.keys():
                item = QtGui.QTreeWidgetItem()
                item.setText(0, benchmark)
                self.__d_algorithm_items[datastructure+algorithm].addChild(item)
                self.__d_benchmark_items.update({datastructure+algorithm+benchmark : item})
            if datastructure+algorithm+benchmark+trial_name not in self.__d_trial_name_items.keys():
                item = QtGui.QTreeWidgetItem()
                item.setText(0, trial_name)
                self.__d_benchmark_items[datastructure+algorithm+benchmark].addChild(item)
                self.__d_trial_name_items.update({datastructure+algorithm+benchmark+trial_name : item})
            
            #build last level of tree --> files    
            for criteria in cost_criteria.keys():
                
                criteria_dict = cost_criteria[criteria]
                filename = criteria_dict["filename"]
                item = QtGui.QTreeWidgetItem()
                identification_name = os.path.basename(filename)
                #set information for tree
                item.setText(0, identification_name)
                #change runtime time stamp
                item.setText(1,time.strftime("%H:%M:%S", time.gmtime(float(runtime))))
                self.__d_trial_name_items[datastructure+algorithm+benchmark+trial_name].addChild(item)
                self.__d_file_items.update({identification_name : filename})
                self.__d_file_information.update({filename : dict})
                
                #get parameter information about the algorithm and save them into dictionary for this file
                param_keys = list(set(dict.keys()) & set(PARAMETER_LIST[algorithm].keys()))
                param_names =[]
                param_values = []
                param_names += ["z-Dimension"]
                param_values += [z_dimension]
                for key in param_keys:
                    param_names.append(key)
                    param_values.append(dict[key])
                self.__dtreeparamtext.update({filename : [param_names, param_values]})
                
                #get statistic data for the file and save them
                #seperate mean_samples which need only be displayed for SUM criteria
                char_names = []
                char_values = []
                try:
                    mean_samples = dict["mean_samples"]
                    if mean_samples != None and criteria == "SUM":
                        char_names.append("mean_samples")
                        char_values.append(mean_samples)
                except:
                    pass
                for key in criteria_dict:
                    char_names.append(key)
                    char_values.append(criteria_dict[key])
                self.__dtreeinfotext.update({filename : [char_names, char_values]})
        #sort the complete tree alphabetic       
        self.__d_data_browser.sortItems(0, QtCore.Qt.AscendingOrder)
            
    def __dataview_fit(self):
        """Fits the data explorer column size to content, a pretty print function for user."""
        for col in range(0,self.__d_data_browser.columnCount(),1):
            self.__d_data_browser.resizeColumnToContents(col)
    
    def __datainfo(self):
        """Shows the information about a datafile.
        It reads the selected data file in the browser and shows the information from the XML file.
        """
        #clear the trees
        self.__d_data_char.clear()
        self.__d_data_params.clear()
        item = self.__d_data_browser.currentItem()
        #get filename
        ident = str(item.text(0))
        try:
            filename = self.__d_file_items[ident]
        except KeyError:
            #self.__popUp("No information available!")
            return
        
        #get information from dictionaries
        try:
            char_lists = self.__dtreeinfotext[filename]
            param_lists = self.__dtreeparamtext[filename]
            
            for i in range(0,len(char_lists[0]),1):
                item = QtGui.QTreeWidgetItem()
                item.setText(0,char_lists[0][i].replace("_", " "))
                item.setText(1,char_lists[1][i])
                self.__d_data_char.addTopLevelItem(item)
            for i in range(0,len(param_lists[0]),1):
                item = QtGui.QTreeWidgetItem()
                item.setText(0,param_lists[0][i].replace("_", " "))
                item.setText(1,param_lists[1][i])
                self.__d_data_params.addTopLevelItem(item)
            #fit column size to content
            for i in range(0,self.__d_data_char.columnCount(),1):
                self.__d_data_char.resizeColumnToContents(i)
            for i in range(0,self.__d_data_params.columnCount(),1):
                self.__d_data_params.resizeColumnToContents(i)
            #sort items alphabetic
            self.__d_data_char.sortItems(0, QtCore.Qt.AscendingOrder)
            self.__d_data_params.sortItems(0, QtCore.Qt.AscendingOrder)
        except:
            #self.__popUp("No information available!")
            return
        
    def __popUp(self, warn_text):
        """Displays a dialog with the text given by warn_text.
            - closed by an OK button
            - is used to inform user about a problem
        @param warn_text: the displayed text in the dialog.
        @type warn_text: string
        """
        
        #create dialog, layout, label an button
        popup = QtGui.QDialog(self)
        popup.setWindowTitle("Warning")
        poplayout = QtGui.QVBoxLayout()
        popup.setLayout(poplayout)
        poplabel = QtGui.QLabel()
        poplabel.setText(warn_text)
        popok = QtGui.QPushButton()
        popok.setText("Ok")
        popbl = QtGui.QHBoxLayout()
        popbl.addStretch()
        popbl.addWidget(popok)
        popbl.addStretch()
        poplabel.setFont(self.__font)
        popok.setFont(self.__font)
        poplayout.addWidget(poplabel)
        poplayout.addLayout(popbl)
        #link button with closing event for dialog
        self.connect(popok, QtCore.SIGNAL("clicked()"), popup.close)
        poplabel.setAlignment(QtCore.Qt.AlignCenter)
        popok.setMaximumWidth(50)
        popup.show()

    def __startCalc(self):
        """Prepares calculation of data files and starts a time estimations."""
        #getting process priority
        priotity = self.__radiobutton_group.checkedId()
        self.__generator = Generator(self.__generated_trials, priotity, time_check = True)
        #start time estimation
        self.__generator.start()
        self.__runDialog = QtGui.QProgressDialog("Calculation running...", "Cancel", 0,self.__generator.total, self)
        self.__runDialog.setWindowTitle("DSC")
        self.__runDialog.setValue(0)
        self.__runDialog.show()
        self.__timer = QtCore.QBasicTimer()
        self.__timer.start(100, self)
        return
    
    def __showTimeEstimation(self):
        """Shows dialog with time estimation."""
        #close run dialog
        self.__runDialog.close()
        del(self.__runDialog)
        #time estimation in pretty format
        string = "             Time estimation for calculation\n\n"
        string +="Task \t\t   Time\n"
        for i in range(1,len(self.__generator.printabletimes)+1,1):
            if i < len(self.__generator.printabletimes):
                string += str(i)+"\t\t"+self.__generator.printabletimes[i-1]+"\n"
            else:
                string += "\nTotal time: \t\t"+self.__generator.printabletimes[i-1]
        del(self.__generator)
        #open dialog for time estimation
        try:
            self.__calcDialog.close()
        except AttributeError:
            pass
        #Dialog
        self.__calcDialog = QtGui.QDialog(self)
        self.__calcDialog.setWindowTitle("Time estimation")
        calcLayout = QtGui.QVBoxLayout()
        self.__calcDialog.setLayout(calcLayout)
        #Label
        dlabel = QtGui.QLabel()                            
        dlabel.setText(string)
        calcLayout.addWidget(dlabel)
        #Buttons
        dblayout = QtGui.QHBoxLayout()
        dbcancel = QtGui.QPushButton()
        dbcancel.setText("Cancel")
        dbok = QtGui.QPushButton()
        dbok.setText("Run")
        dblayout.addWidget(dbcancel)
        dblayout.addWidget(dbok)
        calcLayout.addLayout(dblayout)
        #connecting buttons
        self.connect(dbcancel, QtCore.SIGNAL("clicked()"), self.__calcDialog.close) 
        self.connect(dbok, QtCore.SIGNAL("clicked()") , self.__runCalc)
        self.__calcDialog.show()
        self.__statusbar.showMessage("Time estimated")
        
    def __runCalc(self):
        """Runs calculation in separate thread, shows the process bar and starts the event timer."""
        #new Generator without time_check
        priotity = self.__radiobutton_group.checkedId()
        self.__generator = Generator(self.__generated_trials, priotity, time_check = False)
        #close Dialog
        self.__calcDialog.close()
        self.__statusbar.showMessage("Calculation started")
        #start thread
        self.__generator.start()
        #show run dialog
        self.__runDialog = QtGui.QProgressDialog("Calculation running...", "Cancel", 0,self.__generator.total, self)
        self.__runDialog.setWindowTitle("DSC")
        self.__runDialog.setValue(0)
        self.__runDialog.show()
        #starts event timer
        self.__timer = QtCore.QBasicTimer()
        self.__timer.start(500, self)

    def __finishCalc(self):
        """Writes new content into XML file and clears the planned trials table."""
        #write data to XML file
        #uses the same dictionary form as generated trials but with additional data from generator
        self.__runDialog.setLabelText("Writing new datafiles to database")
        data_to_write = self.__generator.todo
        dict = {}
        #clear planer table
        self.__p_planned_trials.clear()
        for index in data_to_write.keys():
            dict.update({index : {}})
            dict[index].update(data_to_write[index]["individual_params"])
            dict[index].update(data_to_write[index]["init_params"])
        #write new data to XML
        parser = XMLParser(self.__dataxml, "datafile")    
        parser.writeNewData(dict)
        #clear memory to avoid overlapping of different runs
        del(dict)
        del self.__generator
        del self.__generated_trials
        self.__runDialog.close()
        self.__statusbar.showMessage("Calculation finished")
        #refresh data browser
        self.__refreshBrowser()
                                  
    def __evaluate(self, buttonname):
        """Shows a dialog to adjust diagram option.
            - prepares the evaluation
            - supports only diagrams at the moment
        @param buttonname: name of the clicked evaluation button
        @type buttonname: string 
        """
        #Attention:
        #The evaluation part of the program is beta version only
        #This function must be re-designed as the function __showDiagram()
        
        #getting selected files
        selected_entries = self.__d_data_browser.selectedItems()
        self.__evaluable_entries = []
        for item in selected_entries:
            if item.childCount() == 0:
                self.__evaluable_entries.append(item)
                
        if len(self.__evaluable_entries) == 0:
            self.__popUp("No evaluable entries selected!")
            return
        
        #show dialog for diagram adjustment
        dialog = QtGui.QDialog(self.__central)
        dialog.setWindowTitle("Choose diagram options")
        layout = QtGui.QVBoxLayout()
        dialog.setLayout(layout)
        tabs = QtGui.QTabWidget()
        layout.addWidget(tabs)
        
        #create dialog tabs and options
        button_layout = QtGui.QHBoxLayout()
        ok = QtGui.QPushButton("Ok")
        ok.setMaximumWidth(150)
        cancel = QtGui.QPushButton("Cancel")
        cancel.setMaximumWidth(150)
        button_layout.addWidget(ok)
        button_layout.addWidget(cancel)
        layout.addLayout(button_layout)
        self.connect(cancel, QtCore.SIGNAL("clicked ()"), dialog.close)
        self.connect(ok, QtCore.SIGNAL("clicked ()"), self.__showDiagram)
        self.connect(ok, QtCore.SIGNAL("clicked ()"), dialog.close)
        
        self.__d_diagramoptions = {}
        
        def create_structure(tab_entry, layout, saving_key):
            if type(tab_entry) == dict:
                for key in tab_entry.keys():
                    box = QtGui.QGroupBox(key)
                    box_layout = QtGui.QHBoxLayout()
                    box.setLayout(box_layout)
                    layout.addWidget(box)
                    create_structure(tab_entry[key], box_layout, saving_key+key)        
            elif type(tab_entry) == list:
                datatype = tab_entry[0]
                widget = eval(DATATYPE_LIST[datatype])
                widget.setToolTip(tab_entry[1])
                layout.addWidget(widget)
                self.__d_diagramoptions.update({saving_key : widget})
                function_list = LIMITFUNCTION_LIST[datatype]
                i = 0
                for i in range(0,len(function_list),1):
                    eval("widget."+function_list[i]+"(tab_entry[i+2])")
            else:
                self.__popUp("Warning! Check %s in config file." % (tab))
            
        for tab in DIAGRAM_PARAMETER_LIST.keys():
            widget = QtGui.QWidget()
            layout = QtGui.QVBoxLayout()
            widget.setLayout(layout)
            tabs.addTab(widget, tab)
            create_structure(DIAGRAM_PARAMETER_LIST[tab], layout, tab)
             
        self.__eval_func = EVALUATION_PLOT_LIST[buttonname][0]
        dialog.show()
    
    def __showDiagram(self):
        """Calls a evaluation function which shows a diagram."""
        #read out the parameters for diagram adjustment
        #read out information from the XML file about the given files
        filenames = []
        information = {}
        for item in self.__evaluable_entries:
            ident = str(item.text(0))
            filename = self.__d_file_items[ident]
            filenames.append(filename)
            information.update({filename : self.__d_file_information[filename]})
            
        parameter = {} 
        def load_parameter(tab_entry, load_key, current_key, sav_dict):
            if type(tab_entry) == dict:
                for key in tab_entry.keys():
                    if type(tab_entry[key]) == dict:
                        sav_dict.update({key : {}})
                        load_parameter(tab_entry[key], load_key+key, key, sav_dict[key])
                    else:
                        load_parameter(tab_entry[key], load_key+key, key, sav_dict)        
            elif type(tab_entry) == list:
                datatype = tab_entry[0]
                function = READFUNCTION_LIST[datatype] 
                widget = self.__d_diagramoptions[load_key]
                param = eval(datatype+"(widget."+function+"())") 
                sav_dict.update({current_key : param})
            else:
                self.__popUp("Warning! Check %s in config file." % (tab))
        
        for tab in DIAGRAM_PARAMETER_LIST.keys():
            parameter.update({tab : {}})
            load_parameter(DIAGRAM_PARAMETER_LIST[tab], tab, tab, parameter[tab])      
        #try to create diagram
        #try:
        file = self.__eval_func(filenames, information,  parameter, create_png = True)
        #except:
        #    self.__popUp("Failure!")
        #    return
        #show diagram in a dialog with scolbars
        graphic_dialog = QtGui.QDialog(self.__central)
        pixmap = QtGui.QPixmap(FFPLOTS+"/"+file)
        graphic_view = QtGui.QGraphicsView()
        graphic_scene = QtGui.QGraphicsScene()
        graphic_scene.addPixmap(pixmap)
        graphic_view.setScene(graphic_scene)
        graphic_dialog_layout = QtGui.QHBoxLayout()
        graphic_dialog_layout.addWidget(graphic_view)
        graphic_dialog.setLayout(graphic_dialog_layout)
        graphic_dialog.show()
    
    '''Just for old files       
    def __import(self, path):
        """__import(path)
        - gets an file name with path
        - tries to extract information from the filename about the trial
        - user must check the extracted information for correctness
        
        Attention:
        This function will be erased from later versions and is only for import old files.
        """
        #split filename
        for key in self.__m_inputElements.keys():
            self.__m_inputElements[key].clear()
            
        self.__m_inputElements["filename"].setText(path)
        file = os.path.split(path)[1]
        name = os.path.splitext(file)[0]
        list = name.split("-")
        for word in list:
            if word.isupper():
                list[list.index(word)] = word.strip()
                continue
            list[list.index(word)] = word.replace("_", " ").title().strip()
        #defines the extract function
        def tryImport(list):
            i = 0
            for key in OPTIMIZATION_LIST.keys():
                if key in list:
                    self.__m_inputElements["algorithm"].setText(key)
                    i +=1
                    break
                
            for key in BENCHMARK_LIST.keys():
                key = key.rstrip(".yal").capitalize()
                if key in list:
                    key = key.lower()+".yal"
                    self.__m_inputElements["benchmark"].setText(key)
                    i +=1
                    break
                
            for key in DATA_STRUCTURE_LIST.keys():
                nkey = key.replace("-"," ")
                if nkey in list:
                    self.__m_inputElements["datastructure"].setText(key)
                    i +=1
                    break
                nkey = key.replace("-","")
                nkey = nkey.title()
                if nkey in list:
                    self.__m_inputElements["datastructure"].setText(key)
                    i +=1
                    break
                
            for key in COST_LIST.keys():
                if key in list:
                    self.__m_inputElements["cost criteria"].setText(key)
                    i +=1
                    break
                
            for word in list:
                if "Samples" in word:
                    self.__m_inputElements["samples"].setText(word.strip("Samples"))
            return i 
        #if import did not work try another split method              
        if tryImport(list) < 3:
            for key in self.__m_inputElements.keys():
                self.__m_inputElements[key].clear()
            self.__m_inputElements["filename"].setText(path)
            list = name.split(" ")
            list[0] = list.pop(0)+ " " +list.pop(0)
            for word in list:
                if word.isupper():
                    continue
                list[list.index(word)] = word.title()
                if word.isdigit():
                    list[list.index(word)] = word + " Samples"
            if tryImport(list) < 3:
                for key in self.__m_inputElements.keys():
                    self.__m_inputElements[key].clear()

    def __updateXML(self):
        """__updateXML()
        - writes the data from the managing tab into the XML file
        
        Attention:
        This function will be erased from later versions and is only for import old files.
        """
        
        #generate statistic data
        self.__m_pushbuttons["Characteristics"].click()
        #getting information from managing tab
        dict = {}
        for key in self.__m_inputElements.keys():
            if key == "filename" or key == "runtime":
                continue
            if str(self.__m_inputElements[key].text()) == "" and key != "z-D":
                self.__popUp("Missing parameters")
                return
            if str(self.__m_inputElements[key].text()) == "" and key == "z-D":
                self.__m_inputElements[key].setText("auto")
            dict.update({key.replace(" ", "_"): str(self.__m_inputElements[key].text())})
            if key == "cost criteria":
                innerdict = {str(self.__m_inputElements[key].text()): {}}
                dict.update({key.replace(" ", "_"): innerdict})
        
        for key in self.__m_charact_labels.keys():
            tag = key.replace(" ", "_")
            value = str(self.__m_charact_labels[key].text())
            dict["cost_criteria"][str(self.__m_inputElements["cost criteria"].text())].update({tag : value})
        #write data
        filename = str(self.__m_inputElements["filename"].text())
        filename = os.path.split(filename)[1]
        dict["cost_criteria"][str(self.__m_inputElements["cost criteria"].text())].update({"filename": filename})
        filename = str(self.__m_inputElements["runtime"].text())
        dict["cost_criteria"][str(self.__m_inputElements["cost criteria"].text())].update({"runtime": filename})
        XMLParser(self.__dataxml).writeNewData({0:dict}, "datafile")
        
        self.__refreshBrowser()
    '''                        
    def __gen_info_func(self, viewwidget, LOOK_IN_LIST, position):
        """Defines a dynamic function for the call of __infoText(string)
            - gets the text of the current item of the viewwidget (Table, Tree, List)
            - gets an dictionary from config --> text of item must be key in this dict
            - position defines the position of the infoText in list in config dict
        @attention: Usage only for viewwidgets and in planer area tab !!
        
        @param viewwidget: the widget, where the information is displayed
        @type viewwidget: QWidget
        
        @param LOOK_IN_LIST: the list, where the information can be found
        @type LOOK_IN_LIST: list
        
        @param position: the position of the information in LOOK_IN_LIST
        @type position: int
        """
        def click():
            text = str(viewwidget.currentItem().text()) 
            self.__infoText(str(LOOK_IN_LIST[text][position]))
        return click
            
    def __gen_eval_func(self, string):
        """Defines a dynamic function for the call of __evaluate(string).
        @param string: this string is passed over to __evaluate()
        @type string: string
        """
        a = string
        def click():
            self.__evaluate(a)
        return click
    
    def __gen_index_change_func(self, string):
        """Defines a dynamic function for the call of plannedChangeIndex(string)
        @param string: this string is passed over to __plannedChangeIndex(string)
        @type string: string
        """
        a = string
        def index_change():
            self.__plannedChangeIndex(a)
        return index_change
    
if __name__ == "__main__":
    dsc = DSC()
