"""implements a class for the calculation of new data sets by the GUI

This module is used to run the optimization algorithms.
It provides the class Generator and starts a new thread.
The priority of the thread can be set by an global function in this module.
(needed modules: win32api, win32process ,win32con)
The interface between the Generator and the DSC is based on dictionaries.

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""
import threading
from time import gmtime,strftime,clock
from numpy import float64, mean
from random import random
from dsc_suite.ds.data_structure import COST_CRITERIA_LIST
from dsc_suite.gui.config import DATA_STRUCTURE_LIST, BENCHMARK_LIST, OPTIMIZATION_LIST
from dsc_suite.analyses.characteristics import get_statistic_characteristics

'''The following code was taken from
http://code.activestate.com/recipes/496767-set-process-priority-in-windows/
and was greated by Bryan Niederberger.
'''
def setpriority(pid=None,priority=1):
    """ Sets The Priority of a Windows Process.
        Priority is a value between 0-5 where
        2 is normal priority.  Default sets the priority of the current
        python process but can take any valid process ID.
        Recipe 496767 from ActiveState Code
        http://code.activestate.com/recipes/496767-set-process-priority-in-windows/
        and was greated by Bryan Niederberger."""
    import win32api,win32process,win32con  
    priorityclasses = [win32process.IDLE_PRIORITY_CLASS,
                       win32process.BELOW_NORMAL_PRIORITY_CLASS,
                       win32process.NORMAL_PRIORITY_CLASS,
                       win32process.ABOVE_NORMAL_PRIORITY_CLASS,
                       win32process.HIGH_PRIORITY_CLASS,
                       win32process.REALTIME_PRIORITY_CLASS]
    if pid == None:
        pid = win32api.GetCurrentProcessId()
    handle = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, True, pid)
    win32process.SetPriorityClass(handle, priorityclasses[priority])


class Generator(threading.Thread):
    """Is a class which allows the calculation of the data in a new thread.
    This class inherits the class Thread from the Python module threading.
    It must override the function run() to can be executed as a separate thread.
    This is necessary, because the GUI would freeze during the calculation and the OS will offer the termination of the GUI.
    The Generator class is just a box object for separately executed Python code.
    It starts the calculation of the given trials which are generated with the GUI.
    It defines the necessary functions for the algorithms and collects the information about the created data files.
    
    @cvar LockObject: A lock object to avoid the access of parameters by two threads at the same time.
    @type LockObject: thread.lock
    
    @ivar todo: Is the dictionary with the information about the trials.
    For detailed description see constructor.
    @type todo: dictionary
    
    @ivar total: Is the amount of trials in the todo dictionary.
    It can be used for a progress bar.
    @type total: int
    
    @ivar current: Is the current trial which is calulated.
    It can be used for a progress bar.
    @type current: int
    
    @ivar times: Is just a temporary list.
    @type times: list
    
    @ivar printabletimes: Contains the times for the trials in todo.
    It can be used to display the time estimation.
    @type printabletimes: list
    
    @ivar time_check: Is a flag for time estimation run or real calculation.
        - True: time estimation run
        - False: real calculation
    @type time_check: boolean
    """
    
    #LockObject to avoid memory problems
    LockObject = threading.Lock()
    
    def __init__(self, dict, priority, time_check):
        """Prepares the creation of new datafiles in a new thread.
        This is a constructor which sets the needed variables. It is based on a special dictionary structure.
        @param dict: Contains the information about the planned trials which are meant to be calculated.
        
        The keys are:
            - init_params
            - individual_params
        
        Both keys reference another dictionary. Those dictionaries contains the information about the trials.
        They are separated, because there are paramters which have the same name for all algorithms (init_params) such as
        data structure, benchmark, z-Dimension etc. The individual parameters can be looked up in config.py PARAMETER_LIST
        @type dict: dictionary
        
        @param priority: Sets the priority of the thread.
            - 0 IDLE
            - 1 BELOW NORMAL
            - 2 NORMAL
            - 3 ABOVE NORMAL
            - 4 HIGH
            - 5 REALTIME
        Do not use 3 or higher.
        @type priority: int
        
        @param time_check: Is a flag for time estimation run or real calculation.
            - True: time estimation run
            - False: real calculation
        @type time_check: boolean
        """
        threading.Thread.__init__(self)        
        self.todo = dict
        #for the progress bar in the GUI
        self.total = len(dict)
        self.current = 0
        #to save the time estimations        
        self.times = []
        self.printabletimes = []
        #process priority
        #self.__priority = priority
        self.time_check = time_check
                  
    def run(self):
        """Starts a new thread for the calculation of new datafiles.
        This function starts a new thread. It will read all information from the instance variable todo.
        It creates the functions which are used to evaluate a solution and to create new solutions.
        It will also run a short Monte Carlo search to get a very coarse distribution of the solution space.
        This is only necessary if the statistic data for the combination of data structure and benchmark is missing.
        In the first run, the functions will estimate the runtime of each calculation and saves it in a printable format.
        At the second run, the function gets the filenames of the created datafiles and and calculates the statistic
        characterisitcs for each datafile and  updates the todo dictionary with this information. 
        """
        #setting process priority - windows only
        try:
            setpriority(None, self.__priority)
        except:
            pass
        #for all data in todo list
        for index in self.todo.keys():
            #start time for time estimation
            start = clock()
            #update current index
            Generator.LockObject.acquire()
            self.current = index
            Generator.LockObject.release()
            #get necessary dictionaries
            current_dict = self.todo[index]
            init_params = current_dict["init_params"]
            individual_params = current_dict["individual_params"]
            
            #load benchmark and add z-Dimension
            benchmark = BENCHMARK_LIST[init_params["benchmark"]][0](init_params["benchmark"])
            if benchmark.status['number of dimensions'] != 3:
                if init_params["z-Dimension"] == "auto":
                    benchmark.to_3d()
                else:
                    benchmark.to_3d(init_params["z-Dimension"])
            #combine datastructure with benchmark
            datastructure = DATA_STRUCTURE_LIST[init_params["datastructure"]][0](benchmark)
            #get necessary functions from datastructure
            randomSolution = datastructure.get_random_representation
            try:
                changeFunctions = datastructure.get_operations()
            except:
                if init_params["algorithm"] != "Monte Carlo":
                    raise ValueError
            recombineSolutions = datastructure.merge_representations
            #estimate mean values of cost criteria
            mean_samples = init_params["mean_samples"]
            #for time estimation reduce search for mean values
            if self.time_check and mean_samples != None:
                init_params["mean_samples"] = 1
            iruntime = 0
            if mean_samples != None:
                istart = clock()
                #get possible cost criteria
                calculable_costs = set(COST_CRITERIA_LIST.keys())
                #get to-calculate criteria
                cost_criteria = set(init_params["cost_criteria"].keys())
                #create intersection of possible cost criteria and to-calculate criteria
                intersection_criteria = list(calculable_costs & cost_criteria)
                cost_data = []
                #get random solution and get costs
                for criteria in intersection_criteria:
                    cost_data.append([])
                for i in range(0, init_params["mean_samples"],1):
                    solution = randomSolution()
                    costs = datastructure.cost_evaluation(solution, intersection_criteria)
                    for cost in costs:
                        cost_data[costs.index(cost)].append(cost)
                for criteria in intersection_criteria:
                    data = float64(cost_data[intersection_criteria.index(criteria)])
                    mean_value = data.mean()
                    init_params["cost_criteria"]["SUM"].update({"norm_value_"+criteria : mean_value})
                    iend = clock()
                    #estimate runtime for full run
                    if self.time_check:
                        iruntime = (iend - istart)*mean_samples/init_params["mean_samples"]
                        init_params["mean_samples"] = mean_samples
            else:
                iruntime = 0
            #define cost evaluation function
            def get_costs(rep):
                #get possible cost criteria
                criteria_list = COST_CRITERIA_LIST.keys()
                criteria_list.sort()
                #get to-calculate criteria
                calculable_costs = set(criteria_list)
                #create intersection of possible cost criteria and to-calculate criteria
                cost_criteria = set(init_params["cost_criteria"].keys())
                intersection_criteria = list(calculable_costs & cost_criteria)
                intersection_criteria.sort()
                #calculate costs
                costs = datastructure.cost_evaluation(rep, intersection_criteria)
                sum = 0
                #use weight factors and mean_values for cost function
                for criteria in intersection_criteria:
                    sum += init_params["cost_criteria"]["SUM"]["weight_factor_"+criteria] * \
                            costs[intersection_criteria.index(criteria)] / \
                            init_params["cost_criteria"]["SUM"]["norm_value_"+criteria]
                costs.append(sum)
                return costs
            #define function to get a random function to change a representation
            def changeSolution():
                k = random()
                index = int(k*len(changeFunctions.keys()))
                key = changeFunctions.keys()[index]
                operation = changeFunctions[key][0]
                return operation
            #define function to route the data structure operations through
            def getOperations():
                ops = datastructure.get_operations()
                keys = ops.keys()
                keys.sort()
                operations = []
                for key in keys:
                    operations.append(ops[key][0])
                return operations
            #generate dictionary for algorithms
            functions = {"randomSolution"     : randomSolution,
                         "changeSolution"     : changeSolution,
                         "costEvaluation"     : get_costs,
                         "recombineSolutions" : recombineSolutions,
                         "getOperations"      : getOperations}
            end = clock()

            #adding iteration information for non monte-carlo algorithms
            criteria_list = init_params["cost_criteria"].keys()
            criteria_list.sort()
            if init_params["algorithm"] != "Monte Carlo":
                criteria_list += ["ITER"]
            #setting file information dict
            file_info = {}
            file_info.update({"trial_name" : str(init_params["trial_name"])})
            file_info.update({"file_name_list" : []})
            for key in criteria_list:
                file_info["file_name_list"].append(str(init_params["trial_name"]+" "+key+ " " + init_params["datastructure"] + " " + init_params["benchmark"]))
            #estimate time or do real calculation
            if self.time_check:
                time = OPTIMIZATION_LIST[init_params["algorithm"]][0](functions, individual_params, file_info, self.time_check)
                time += (end-start)+iruntime
                self.times.append(time)
            #includes update of the dictionaries with data for XML
            else:
                #call optimization algorithm
                start = clock()
                files = OPTIMIZATION_LIST[init_params["algorithm"]][0](functions, individual_params, file_info, self.time_check)
                end = clock()
                #update XML information
                init_params.update({"runtime" : (end-start)})
                #connect filenames with criteria                    
                for criteria in criteria_list:
                    try:
                        init_params["cost_criteria"][criteria].update({"filename" : files[criteria_list.index(criteria)]})
                    except KeyError:
                        init_params["cost_criteria"].update({criteria : {"filename" : files[criteria_list.index(criteria)]}})
                    if criteria != "ITER" and init_params["algorithm"] != "EvalOperation Algorithm":
                        result = get_statistic_characteristics(files[criteria_list.index(criteria)])
                        for key in result:
                            init_params["cost_criteria"][criteria].update({key.replace(" ","_") : result[key]})
        #return runtime in pretty print format 
        if self.time_check:
            total_time = sum(self.times)
            for time in self.times:
                self.printabletimes.append(strftime("%H:%M:%S", gmtime(time)))
            self.printabletimes.append(strftime("%H:%M:%S", gmtime(total_time)))
            return
        else:
            return
            