'''implements a Evolutionary Optimization concept

This module implements a Evolutionary Optimization concept. The function can be called either from console or from GUI(recommended).
It saves the solution files in standard folder for data.


@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
'''
import math, os, random
from dsc_suite.tools.toolpaths import FOLDER_FOR_DATA
from time import strftime, time
import numpy

def generate_evolution_data(functions, parameters, file_info, time_check = False):
    '''Optimizes the given floorplan with a Evolutionary Algorithm concept and saves it progress in data files.
    
    This function uses a evolutioary algorithm concept to optimize the cost function of a given floorplan.
    First a initial population (parents) is created using the random solution function from the data structure.
    Second every solution is combined with another solution. That results in many other solutions including the base solutions.
    From that pool a amount of solution is selected randomly an used to create the next generation of solutions.
    This generation of solutions is evaluated. After that a mix from the solutions are taken as the new parent generation.
    
    @param functions: Contains functions for the calculation.
    
    essential keys:
        - "randomSolution" : Returns a random representation.
        - "changeSolution" : Returns a function to change the given representation.
        - "costEvaluation" : Returns the calculated costs for the given representation.
        - "recombineSolutions" : Returns list of new representations created from the two given representations.
    @type functions: dictionary    
    
    @param parameters: Contains algorithm parameters.
    
    essential keys:
        - "parents" : number of parent individuals each generation
        - "children" : number of children individuals each generation
        - "generations" : number of generated generations
    @type parameters: dictionary           
    
    @param file_info: Contains information about the filename of the created file.
    
    essential keys:
        - "trial_name" : name of trial given in GUI
        - "file_name_list" : additional information included in file name (can be empty string)
    @type file_info: dictionary
    
    @param time_check: Sets the function to estimate runtime or full calculation.
        - True: estimates runtime with reduced samples
        - False: full calculation with saving data to files
    @type time_check: boolean
    
    @return: Returns either a list with filenames (time_check == False) or a float with the estimated runtime (time_check == True).
    @rtype: list, float
    '''      
    #psyco should speed python a little bit
    if time_check:
        generations = parameters["generations"]
        parameters["generations"] = 1
    import psyco
    psyco.full()
    #start time estimation
    start = time()
    calculated_rep = []
    rep_costs = []
    #sorting function for evaluation
    def compare_costs(rep_a, rep_b):
        try:
            cost_a = rep_costs[calculated_rep.index(rep_a)]
        except ValueError:
            calculated_rep.append(rep_a)
            cost_a = functions["costEvaluation"](rep_a)
            rep_costs.append(cost_a)
        try:
            cost_b = rep_costs[calculated_rep.index(rep_b)]
        except ValueError:
            calculated_rep.append(rep_b)
            cost_b = functions["costEvaluation"](rep_b)
            rep_costs.append(cost_b)
            
        #return specific value for compare result    
        if cost_a > cost_b:
            return 1
        elif cost_a == cost_b:
            return 0
        else:
            return -1
    #parents of the generation
    parents = []
    #create initial generation
    for i in range(0,parameters["parents"],1):
        parents.append(functions["randomSolution"]())
    j = 0
    cost_data = []
    costs = functions["costEvaluation"](parents[0])
    for cost in costs:
        cost_data.append([])
    cost_data.append([])
    #----------------------MAIN OPTIMIZATION PART-------------------- 
    while j < parameters["generations"]:
        #reset lists for already calculated solutions
        calculated_rep = []
        rep_costs = []
        #save parents
        for rep in parents:
            costs = functions["costEvaluation"](rep)
            for cost in costs:
                index = costs.index(cost)
                cost_data[index].append(cost)
            cost_data[len(cost_data)-1].append(j)
        #create new children      
        children = []
        #recombine all parents
        for i in range(0,len(parents)-1,1):
            for k in range(i+1,len(parents),1):
                #recombination
                new_children = functions["recombineSolutions"](parents[i], parents[k])
                #not enough children, then mutate
                while parameters["children"] > len(new_children):
                    rand = random.random()
                    index = int(rand*len(new_children))
                    new_children.append( functions["changeSolution"]()(new_children[index]))
                #if more then necessary choose randomly 
                for num in range(0,parameters["children"],1):
                    rand = random.random()
                    index = int(rand*len(new_children))
                    children.append(new_children.pop(index))       
        #mutate 5% of children to get genetic fluctuations
        for i in range(0,int(len(children)*0.05),1):
            rand = random.random()
            index = int(rand*len(new_children))
            children[index] = functions["changeSolution"]()(children[index])
        #evaluate children 
        children.sort(compare_costs)
        #pick children to new parent generation
        #pick 50% from best, 35% from middle and 15% from worst
        amount_of_best = int(round(0.6*parameters["parents"],0))
        amount_of_middle= int(round(0.3*parameters["parents"],0))
        amount_of_worse= int(round(0.1*parameters["parents"],0))
        
        parents = children[:amount_of_best]
        temp = children[int(len(children)/2):int(len(children)/2+amount_of_middle)]
        parents += temp
        temp = children[int(len(children)-amount_of_worse):]
        parents += temp
        j += 1
    #------------SAVING ROUTINE---------------------------
    #stop psyco
    end = time()        
    psyco.stop()
    
    #estimate runtime
    runtime = end-start
    #return runtime      
    if time_check:
        parameters["generations"] = generations 
        return runtime*generations
    #save files
    trial_name = file_info["trial_name"]
    file_name_list = file_info["file_name_list"]
    path = trial_name + '/Evolutionary Algorithm/'
    if not os.path.exists(os.path.join(FOLDER_FOR_DATA, path)):
        os.makedirs(os.path.join(FOLDER_FOR_DATA, path))
    filename_list = []
    for list in cost_data:
        file_name_begin = file_name_list[cost_data.index(list)]
        i = 1
        filename = file_name_begin + ' %02i P%i C%i G%i.npy' % (i, parameters["parents"], parameters["children"], parameters["generations"])
        while os.path.exists(os.path.join(FOLDER_FOR_DATA, path, filename)):
            i += 1
            filename = file_name_begin + ' %02i P%i C%i G%i.npy' % (i, parameters["parents"], parameters["children"], parameters["generations"])
        filename_list.append(os.path.join(path, filename))
        if not time_check:
            numpy.save(os.path.join(FOLDER_FOR_DATA, path, filename), numpy.float64(list))
    #return list of filenames    
    return filename_list
