"""implements several statistical basic methods

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""


##### PROBABILITY DISTRIBUTIONS ################################################
class ProbabilityDistributions(object):
    """Implements basic probability distribution functionality.
    This class implements basic probability distribution functionality,
    currently the generation of random distributions. One usage is the
    random benchmark generation.

    Most methods are static (class) methods. Basically, this class is used to
    encapsulated conjoined functions.
    
    @cvar IMPLEMENTED_DISTRIBUTIONS: List with supported distributions.
    @type IMPLEMENTED_DISTRIBUTIONS: list-of-strings
    """

    # static attribute declaring implemented distributions
    IMPLEMENTED_DISTRIBUTIONS = ['uniform',
                                 'gauss',
                                 'exponential',
                                 'half-gauss']
    #TODO: later also chi**2, F-distribution
    # if new methods are implemented, update docstrings in dependent modules
    # (i.e., RandomBench, ...)

    @staticmethod
    def get_distribution(n, method, left, right):
        """Creates a random solutions which fits the given distribution.
        This function generates n random values which fits the given distribution(method).
        The solution space can be limited on both sides.
        Supported distributions:
            - uniform
            - gauss
            - exponential
            - half-gauss
        
        @param n: number of generated values
        @type n: int
        
        @param method: name of the distribution
        @type method: string
        
        @param left: left/lower limit of solution space
        @type left: float
        
        @param right: right/upper limit of solution space
        @type right:
        
        @return: Returns a list with generated solutions.
        @rtype: list-of-floats
        """
        # assume left <= right, otherwise in some cases (i.e. unsymetric distributions,
        # such as half-gauss or exponential) a special inversion version is used
        if right < left:
            left, right = right, left
            inverse = True
        else:
            inverse = False
        assert left >= 0
            
        import random
        from exceptions import ValueError
        
        distribution = []

        if left == right: # sonderfall zuerst behandeln
            def single_random_value():
                return left            
        elif method == 'uniform':
            def single_random_value():
                return random.uniform(left, right)
        elif method == 'gauss':
            mu = (left + right) / 2.0
            sigma = (right - left) / (2.0 * 3) # 3 sigma intervall -> 99.73 % of all values
            def single_random_value():
                value = random.gauss(mu, sigma)
                while value < left or value > right:
                    value = random.gauss(mu, sigma)
                return value
        elif method == 'half-gauss':
            if not inverse:
                mu = left
            else:
                mu = right
            sigma = right - left # sigma intervall -> 68.27 % of all values
            def single_random_value():
                value = random.gauss(mu, sigma)
                # if random value < mu then flip at mu
                if (inverse and value > mu) or (not inverse and value < mu):
                    value = 2 * mu - value
                while (inverse and value < left) or (not inverse and value > right):
                    value = random.gauss(mu, sigma)
                    if (inverse and value > mu) or (not inverse and value < mu):
                        value = 2 * mu - value
                return value
        elif method == 'exponential':
            margin = right - left
            lambd = 2.0 / margin
            if inverse:
                lambd = -lambd
                offset = right
            else:
                offset = left
            def single_random_value():
                value = random.expovariate(lambd)
                while (inverse and value < -margin) or (not inverse and value > margin):
                    value = random.expovariate(lambd)
                return value + offset
        else:
            raise ValueError('%s is not implemented.' % method)

        i = 0
        while i < n:
            distribution.append(single_random_value())
            i += 1
        return distribution
